import os
import os.path
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbproxy import *
import lldbgdbserverutils
import re


class ThreadSnapshot:
    def __init__(self, thread_id, registers):
        self.thread_id = thread_id
        self.registers = registers


class MemoryBlockSnapshot:
    def __init__(self, address, data):
        self.address = address
        self.data = data


class StateSnapshot:
    def __init__(self, thread_snapshots, memory):
        self.thread_snapshots = thread_snapshots
        self.memory = memory
        self.thread_id = None


class RegisterInfo:
    def __init__(self, lldb_index, name, bitsize, little_endian):
        self.lldb_index = lldb_index
        self.name = name
        self.bitsize = bitsize
        self.little_endian = little_endian


BELOW_STACK_POINTER = 16384
ABOVE_STACK_POINTER = 4096

BLOCK_SIZE = 1024

SOFTWARE_BREAKPOINTS = 0
HARDWARE_BREAKPOINTS = 1
WRITE_WATCHPOINTS = 2


class ReverseTestBase(GDBProxyTestBase):
    """
    Base class for tests that need reverse execution.

    This class uses a gdbserver proxy to add very limited reverse-
    execution capability to lldb-server/debugserver for testing
    purposes only.

    To use this class, run the inferior forward until some stopping point.
    Then call `start_recording()` and execute forward again until reaching
    a software breakpoint; this class records the state before each execution executes.
    At that point, the server will accept "bc" and "bs" packets to step
    backwards through the state.
    When executing during recording, we only allow single-step and continue without
    delivering a signal, and only software breakpoint stops are allowed.

    We assume that while recording is enabled, the only effects of instructions
    are on general-purpose registers (read/written by the 'g' and 'G' packets)
    and on memory bytes between [SP - BELOW_STACK_POINTER, SP + ABOVE_STACK_POINTER).
    """

    NO_DEBUG_INFO_TESTCASE = True

    """
    A list of StateSnapshots in time order.

    There is one snapshot per single-stepped instruction,
    representing the state before that instruction was
    executed. The last snapshot in the list is the
    snapshot before the last instruction was executed.
    This is an undo log; we snapshot a superset of the state that may have
    been changed by the instruction's execution.
    """
    snapshots = None
    recording_enabled = False

    breakpoints = None

    pc_register_info = None
    sp_register_info = None
    general_purpose_register_info = None

    def __init__(self, *args, **kwargs):
        GDBProxyTestBase.__init__(self, *args, **kwargs)
        self.breakpoints = [set(), set(), set(), set(), set()]

    def respond(self, packet):
        if not packet:
            raise ValueError("Invalid empty packet")
        if packet == self.server.PACKET_INTERRUPT:
            # Don't send a response. We'll just run to completion.
            return []
        if self.is_command(packet, "qSupported", ":"):
            # Disable multiprocess support in the server and in LLDB
            # since Mac debugserver doesn't support it and we want lldb-server to
            # be consistent with that
            reply = self.pass_through(packet.replace(";multiprocess", ""))
            return reply.replace(";multiprocess", "") + ";ReverseStep+;ReverseContinue+"
        if packet == "c" or packet == "s":
            packet = "vCont;" + packet
        elif (
            packet[0] == "c" or packet[0] == "s" or packet[0] == "C" or packet[0] == "S"
        ):
            raise ValueError(
                "Old-style continuation packets with address or signal not supported yet"
            )
        if self.is_command(packet, "vCont", ";"):
            if self.recording_enabled:
                return self.continue_with_recording(packet)
            snapshots = []
        if packet == "bc":
            return self.reverse_continue()
        if packet == "bs":
            return self.reverse_step()
        if packet == "jThreadsInfo":
            # Suppress this because it contains thread stop reasons which we might
            # need to modify, and we don't want to have to implement that.
            return ""
        if packet[0] == "x":
            # Suppress *binary* reads as results starting with "O" can be mistaken for an output packet
            # by the test server code
            return ""
        if packet[0] == "z" or packet[0] == "Z":
            reply = self.pass_through(packet)
            if reply == "OK":
                self.update_breakpoints(packet)
            return reply
        return GDBProxyTestBase.respond(self, packet)

    def start_recording(self):
        self.recording_enabled = True
        self.snapshots = []

    def stop_recording(self):
        """
        Don't record when executing foward.

        Reverse execution is still supported until the next forward continue.
        """
        self.recording_enabled = False

    def is_command(self, packet, cmd, follow_token):
        return packet == cmd or packet[0 : len(cmd) + 1] == cmd + follow_token

    def update_breakpoints(self, packet):
        m = re.match("([zZ])([01234]),([0-9a-f]+),([0-9a-f]+)", packet)
        if m is None:
            raise ValueError("Invalid breakpoint packet: " + packet)
        t = int(m.group(2))
        addr = int(m.group(3), 16)
        kind = int(m.group(4), 16)
        if m.group(1) == "Z":
            self.breakpoints[t].add((addr, kind))
        else:
            self.breakpoints[t].discard((addr, kind))

    def breakpoint_triggered_at(self, pc):
        if any(addr == pc for addr, kind in self.breakpoints[SOFTWARE_BREAKPOINTS]):
            return True
        if any(addr == pc for addr, kind in self.breakpoints[HARDWARE_BREAKPOINTS]):
            return True
        return False

    def watchpoint_triggered(self, new_value_block, current_contents):
        """Returns the address or None."""
        for watch_addr, kind in self.breakpoints[WRITE_WATCHPOINTS]:
            for offset in range(0, kind):
                addr = watch_addr + offset
                if (
                    addr >= new_value_block.address
                    and addr < new_value_block.address + len(new_value_block.data)
                ):
                    index = addr - new_value_block.address
                    if (
                        new_value_block.data[index * 2 : (index + 1) * 2]
                        != current_contents[index * 2 : (index + 1) * 2]
                    ):
                        return watch_addr
        return None

    def continue_with_recording(self, packet):
        self.logger.debug("Continue with recording enabled")

        step_packet = "vCont;s"
        if packet == "vCont":
            requested_step = False
        else:
            m = re.match("vCont;(c|s)(.*)", packet)
            if m is None:
                raise ValueError("Unsupported vCont packet: " + packet)
            requested_step = m.group(1) == "s"
            step_packet += m.group(2)

        while True:
            snapshot = self.capture_snapshot()
            reply = self.pass_through(step_packet)
            (stop_signal, stop_pairs) = self.parse_stop_reply(reply)
            if stop_signal != 5:
                raise ValueError("Unexpected stop signal: " + reply)
            is_swbreak = False
            thread_id = None
            for key, value in stop_pairs.items():
                if key == "thread":
                    thread_id = self.parse_thread_id(value)
                    continue
                if re.match("[0-9a-f]+", key):
                    continue
                if key == "swbreak" or (key == "reason" and value == "breakpoint"):
                    is_swbreak = True
                    continue
                if key == "metype":
                    reason = self.stop_reason_from_mach_exception(stop_pairs)
                    if reason == "breakpoint":
                        is_swbreak = True
                    elif reason != "singlestep":
                        raise ValueError(f"Unsupported stop reason in {reply}")
                    continue
                if key in [
                    "name",
                    "threads",
                    "thread-pcs",
                    "reason",
                    "mecount",
                    "medata",
                    "memory",
                ]:
                    continue
                raise ValueError(f"Unknown stop key '{key}' in {reply}")
            if is_swbreak:
                self.logger.debug("Recording stopped")
                return reply
            if thread_id is None:
                return ValueError("Expected thread ID: " + reply)
            snapshot.thread_id = thread_id
            self.snapshots.append(snapshot)
            if requested_step:
                self.logger.debug("Recording stopped for step")
                return reply

    def stop_reason_from_mach_exception(self, stop_pairs):
        # See StopInfoMachException::CreateStopReasonWithMachException.
        if int(stop_pairs["metype"]) != 6:  # EXC_BREAKPOINT
            raise ValueError(f"Unsupported exception type {value} in {reply}")
        medata = stop_pairs["medata"]
        arch = self.getArchitecture()
        if arch in ["amd64", "i386", "x86_64"]:
            if int(medata[0], 16) == 2:
                return "breakpoint"
            if int(medata[0], 16) == 1 and int(medata[1], 16) == 0:
                return "singlestep"
        elif arch in ["arm64", "arm64e"]:
            if int(medata[0], 16) == 1 and int(medata[1], 16) != 0:
                return "breakpoint"
            elif int(medata[0], 16) == 1 and int(medata[1], 16) == 0:
                return "singlestep"
        else:
            raise ValueError(f"Unsupported architecture '{arch}'")
        raise ValueError(f"Unsupported exception details in {reply}")

    def parse_stop_reply(self, reply):
        if not reply:
            raise ValueError("Invalid empty packet")
        if reply[0] == "T" and len(reply) >= 3:
            result = {}
            for k, v in self.parse_pairs(reply[3:]):
                if k in ["medata", "memory"]:
                    if k in result:
                        result[k].append(v)
                    else:
                        result[k] = [v]
                else:
                    result[k] = v
            return (int(reply[1:3], 16), result)
        raise ValueError("Unsupported stop reply: " + reply)

    def parse_pairs(self, text):
        for pair in text.split(";"):
            if not pair:
                continue
            m = re.match("([^:]+):(.*)", pair)
            if m is None:
                raise ValueError("Invalid pair text: " + text)
            yield (m.group(1), m.group(2))

    def capture_snapshot(self):
        """Snapshot all threads and their stack memories."""
        self.ensure_register_info()
        current_thread = self.get_current_thread()
        thread_snapshots = []
        memory = []
        for thread_id in self.get_thread_list():
            registers = {}
            for index in sorted(self.general_purpose_register_info.keys()):
                reply = self.pass_through(f"p{index:x};thread:{thread_id:x};")
                if reply == "" or reply[0] == "E":
                    # Mac debugserver tells us about registers that it won't let
                    # us actually read. Ignore those registers.
                    self.logger.debug(f"Failed to read register {index:x}")
                    continue
                registers[index] = reply
            thread_snapshot = ThreadSnapshot(thread_id, registers)
            thread_sp = self.get_register(
                self.sp_register_info, thread_snapshot.registers
            )

            # The memory above or below the stack pointer may be mapped, but not
            # both readable and writeable. For example on Arm 32-bit Linux, there
            # is a "[vectors]" mapping above the stack, which can be read but not
            # written to.
            #
            # Therefore, we should limit any reads to the stack region, which we
            # know is readable and writeable.
            region_info = self.get_memory_region_info(thread_sp)
            lower = max(thread_sp - BELOW_STACK_POINTER, region_info["start"])
            upper = min(
                thread_sp + ABOVE_STACK_POINTER,
                region_info["start"] + region_info["size"],
            )

            memory += self.read_memory(lower, upper)
            thread_snapshots.append(thread_snapshot)
        self.set_current_thread(current_thread)
        return StateSnapshot(thread_snapshots, memory)

    def restore_snapshot(self, snapshot):
        """
        Restore the snapshot during reverse execution.

        If this triggers a breakpoint or watchpoint, return the stop reply,
        otherwise None.
        """
        current_thread = self.get_current_thread()
        stop_reasons = []
        for thread_snapshot in snapshot.thread_snapshots:
            thread_id = thread_snapshot.thread_id
            for lldb_index in sorted(thread_snapshot.registers.keys()):
                data = thread_snapshot.registers[lldb_index]
                reply = self.pass_through(
                    f"P{lldb_index:x}={data};thread:{thread_id:x};"
                )
                if reply != "OK":
                    try:
                        reg_name = self.general_purpose_register_info[lldb_index].name
                    except KeyError:
                        reg_name = f"with index {lldb_index}"
                    raise ValueError(f"Can't restore thread register {reg_name}")
            if thread_id == snapshot.thread_id:
                new_pc = self.get_register(
                    self.pc_register_info, thread_snapshot.registers
                )
                if self.breakpoint_triggered_at(new_pc):
                    stop_reasons.append([("reason", "breakpoint")])
        self.set_current_thread(current_thread)
        for block in snapshot.memory:
            current_memory = self.pass_through(
                f"m{block.address:x},{(len(block.data)//2):x}"
            )
            if not current_memory or current_memory[0] == "E":
                raise ValueError("Can't read back memory")
            reply = self.pass_through(
                f"M{block.address:x},{len(block.data)//2:x}:" + block.data
            )
            if reply != "OK":
                raise ValueError(
                    f"Can't restore memory block ranging from 0x{block.address:x} to 0x{block.address+len(block.data):x}."
                )
            watch_addr = self.watchpoint_triggered(block, current_memory)
            if watch_addr is not None:
                stop_reasons.append(
                    [("reason", "watchpoint"), ("watch", f"{watch_addr:x}")]
                )
        if stop_reasons:
            pairs = ";".join(f"{key}:{value}" for key, value in stop_reasons[0])
            return f"T05thread:{snapshot.thread_id:x};{pairs};"
        return None

    def reverse_step(self):
        if not self.snapshots:
            self.logger.debug("Reverse-step at history boundary")
            return self.history_boundary_reply(self.get_current_thread())
        self.logger.debug("Reverse-step started")
        snapshot = self.snapshots.pop()
        stop_reply = self.restore_snapshot(snapshot)
        self.set_current_thread(snapshot.thread_id)
        self.logger.debug("Reverse-step stopped")
        if stop_reply is None:
            return self.singlestep_stop_reply(snapshot.thread_id)
        return stop_reply

    def reverse_continue(self):
        self.logger.debug("Reverse-continue started")
        thread_id = None
        while self.snapshots:
            snapshot = self.snapshots.pop()
            stop_reply = self.restore_snapshot(snapshot)
            thread_id = snapshot.thread_id
            if stop_reply is not None:
                self.set_current_thread(thread_id)
                self.logger.debug("Reverse-continue stopped")
                return stop_reply
        if thread_id is None:
            thread_id = self.get_current_thread()
        else:
            self.set_current_thread(snapshot.thread_id)
        self.logger.debug("Reverse-continue stopped at history boundary")
        return self.history_boundary_reply(thread_id)

    def get_current_thread(self):
        reply = self.pass_through("qC")
        return self.parse_thread_id(reply[2:])

    def parse_thread_id(self, thread_id):
        m = re.match("([0-9a-f]+)", thread_id)
        if m is None:
            raise ValueError("Invalid thread ID: " + thread_id)
        return int(m.group(1), 16)

    def history_boundary_reply(self, thread_id):
        return f"T00thread:{thread_id:x};replaylog:begin;"

    def singlestep_stop_reply(self, thread_id):
        return f"T05thread:{thread_id:x};"

    def set_current_thread(self, thread_id):
        """
        Set current thread in inner gdbserver.
        """
        if thread_id >= 0:
            self.pass_through(f"Hg{thread_id:x}")
            self.pass_through(f"Hc{thread_id:x}")
        else:
            self.pass_through(f"Hc-1")
            self.pass_through(f"Hg-1")

    def get_register(self, register_info, registers):
        if register_info.bitsize % 8 != 0:
            raise ValueError("Register size must be a multiple of 8 bits")
        if register_info.lldb_index not in registers:
            raise ValueError("Register value not captured")
        data = registers[register_info.lldb_index]
        num_bytes = register_info.bitsize // 8
        bytes = []
        for i in range(0, num_bytes):
            bytes.append(int(data[i * 2 : (i + 1) * 2], 16))
        if register_info.little_endian:
            bytes.reverse()
        result = 0
        for byte in bytes:
            result = (result << 8) + byte
        return result

    def get_memory_region_info(self, addr):
        reply = self.pass_through(f"qMemoryRegionInfo:{addr:x}")
        if not reply or reply[0] == "E":
            raise RuntimeError("Failed to get memory region info.")

        # Valid reply looks like:
        # start:fffcf000;size:21000;permissions:rw;flags:;name:5b737461636b5d;
        values = [v for v in reply.strip().split(";") if v]
        region_info = {}
        for value in values:
            key, value = value.split(
                ":",
            )
            region_info[key] = value

        if not ("start" in region_info and "size" in region_info):
            raise RuntimeError("Did not get extent of memory region.")

        region_info["start"] = int(region_info["start"], 16)
        region_info["size"] = int(region_info["size"], 16)

        return region_info

    def read_memory(self, start_addr, end_addr):
        """
        Read a region of memory from the target.

        Some of the addresses may extend into memory we cannot read, skip those.

        Return a list of blocks containing the valid area(s) in the
        requested range.
        """
        regions = []
        start_addr = start_addr - (start_addr % BLOCK_SIZE)
        if end_addr % BLOCK_SIZE > 0:
            end_addr = end_addr - (end_addr % BLOCK_SIZE) + BLOCK_SIZE
        for addr in range(start_addr, end_addr, BLOCK_SIZE):
            reply = self.pass_through(f"m{addr:x},{(BLOCK_SIZE - 1):x}")
            if reply and reply[0] != "E":
                block = MemoryBlockSnapshot(addr, reply)
                regions.append(block)
        return regions

    def ensure_register_info(self):
        if self.general_purpose_register_info is not None:
            return
        reply = self.pass_through("qHostInfo")
        little_endian = any(
            kv == ("endian", "little") for kv in self.parse_pairs(reply)
        )
        self.general_purpose_register_info = {}
        lldb_index = 0
        while True:
            reply = self.pass_through(f"qRegisterInfo{lldb_index:x}")
            if not reply or reply[0] == "E":
                break
            info = {k: v for k, v in self.parse_pairs(reply)}
            reg_info = RegisterInfo(
                lldb_index, info["name"], int(info["bitsize"]), little_endian
            )
            if (
                info["set"] == "General Purpose Registers"
                and not "container-regs" in info
            ):
                self.general_purpose_register_info[lldb_index] = reg_info
            if "generic" in info:
                if info["generic"] == "pc":
                    self.pc_register_info = reg_info
                elif info["generic"] == "sp":
                    self.sp_register_info = reg_info
            lldb_index += 1
        if self.pc_register_info is None or self.sp_register_info is None:
            raise ValueError("Can't find generic pc or sp register")

    def get_thread_list(self):
        threads = []
        reply = self.pass_through("qfThreadInfo")
        while True:
            if not reply:
                raise ValueError("Missing reply packet")
            if reply[0] == "m":
                for id in reply[1:].split(","):
                    threads.append(self.parse_thread_id(id))
            elif reply[0] == "l":
                return threads
            reply = self.pass_through("qsThreadInfo")



"""
Test more expression command sequences with objective-c.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationTestCaseNSError(TestBase):
    @expectedFailureAll(archs=["i[3-6]86"], bugnumber="<rdar://problem/28814052>")
    def test_runtime_types(self):
        """Test commands that require runtime types"""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Break here for NSString tests", lldb.SBFileSpec("main.m", False)
        )

        # Test_NSString:
        self.runCmd("thread backtrace")
        self.expect("expression [str length]", patterns=[r"\(NSUInteger\) \$.* ="])
        self.expect("expression str.length")
        self.expect('expression str = [NSString stringWithCString: "new"]')
        self.expect(
            'po [NSError errorWithDomain:@"Hello" code:35 userInfo:@{@"NSDescription" : @"be completed."}]',
            substrs=["Error Domain=Hello", "Code=35", "be completed."],
        )
        self.runCmd("process continue")

    @expectedFailureAll(archs=["i[3-6]86"], bugnumber="<rdar://problem/28814052>")
    def test_NSError_p(self):
        """Test that p of the result of an unknown method does require a cast."""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.m", False)
        )
        self.expect(
            "expression [NSError thisMethodIsntImplemented:0]",
            error=True,
            patterns=[
                "no known method",
                "cast the message send to the method's return type",
            ],
        )
        self.runCmd("process continue")

    @skipIfOutOfTreeDebugserver
    def test_runtime_types_efficient_memreads(self):
        # Test that we use an efficient reading of memory when reading
        # Objective-C method descriptions.
        logfile = os.path.join(self.getBuildDir(), "log.txt")
        self.runCmd(f"log enable -f {logfile} gdb-remote packets process")
        self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Break here for NSString tests", lldb.SBFileSpec("main.m", False)
        )

        self.runCmd(f"proc plugin packet send StartTesting", check=False)
        self.expect('expression str = [NSString stringWithCString: "new"]')
        self.runCmd(f"proc plugin packet send EndTesting", check=False)

        self.assertTrue(os.path.exists(logfile))
        log_text = open(logfile).read()
        log_text = log_text.split("StartTesting", 1)[-1].split("EndTesting", 1)[0]

        # This test is only checking that the packet it used at all (and that
        # no errors are produced). It doesn't check that the packet is being
        # used to solve a problem in an optimal way.
        self.assertIn("MultiMemRead:", log_text)
        self.assertNotIn("MultiMemRead error", log_text)



"""
Tests that TSan correctly reports the filename and line number of a racy global C++ variable.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import json


class TsanCPPGlobalLocationTestCase(TestBase):
    @expectedFailureNetBSD
    @skipIfLinux  # https://github.com/llvm/llvm-project/issues/191012
    @skipIfFreeBSD  # llvm.org/pr21136 runtimes not yet available by default
    @skipIfRemote
    @skipUnlessThreadSanitizer
    def test(self):
        self.build()
        self.tsan_tests()

    def tsan_tests(self):
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.runCmd("run")

        stop_reason = (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        )
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        # the stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            "A data race should be detected",
            substrs=["stopped", "stop reason = Data race detected"],
        )

        self.expect(
            "thread info -s",
            "The extended stop info should contain the TSan provided fields",
            substrs=["instrumentation_class", "description", "mops"],
        )

        output_lines = self.res.GetOutput().split("\n")
        json_line = "\n".join(output_lines[2:])
        data = json.loads(json_line)
        self.assertEqual(data["instrumentation_class"], "ThreadSanitizer")
        self.assertEqual(data["issue_type"], "data-race")

        self.assertTrue(data["location_filename"].endswith("/main.cpp"))
        self.assertEqual(
            data["location_line"], line_number("main.cpp", "// global variable")
        )



"""
Test setting a breakpoint by file and line when many instances of the
same file name exist in the CU list.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBreakpointSameCU(TestBase):
    def test_breakpoint_same_cu(self):
        self.build()
        target = self.createTestTarget()

        # Break both on the line before the code:
        comment_line = line_number("common.cpp", "// A comment here")
        self.assertNotEqual(comment_line, 0, "line_number worked")
        bkpt = target.BreakpointCreateByLocation("common.cpp", comment_line)
        self.assertEqual(
            bkpt.GetNumLocations(), 4, "Got the right number of breakpoints"
        )

        # And break on the code, both should work:
        code_line = line_number("common.cpp", "// The line with code")
        self.assertNotEqual(comment_line, 0, "line_number worked again")
        bkpt = target.BreakpointCreateByLocation("common.cpp", code_line)
        self.assertEqual(
            bkpt.GetNumLocations(), 4, "Got the right number of breakpoints"
        )



"""
When using C++11 in place member initialization, show that we
can set and hit breakpoints on initialization lines.  This is a
little bit tricky because we try not to move file and line breakpoints 
across function boundaries but these lines are outside the source range
of the constructor.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestCase(TestBase):
    def test_breakpoints_on_initializers(self):
        """Show we can set breakpoints on initializers appearing both before
        and after the constructor body, and hit them."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.first_initializer_line = line_number(
            "main.cpp", "Set the before constructor breakpoint here"
        )
        self.second_initializer_line = line_number(
            "main.cpp", "Set the after constructor breakpoint here"
        )

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, " Set a breakpoint here to get started", self.main_source_file
        )

        # Now set breakpoints on the two initializer lines we found in the test startup:
        bkpt1 = target.BreakpointCreateByLocation(
            self.main_source_file, self.first_initializer_line
        )
        self.assertEqual(bkpt1.GetNumLocations(), 1)
        bkpt2 = target.BreakpointCreateByLocation(
            self.main_source_file, self.second_initializer_line
        )
        self.assertEqual(bkpt2.GetNumLocations(), 1)

        # Now continue, we should stop at the two breakpoints above, first the one before, then
        # the one after.
        self.assertEqual(
            len(lldbutil.continue_to_breakpoint(process, bkpt1)),
            1,
            "Hit first breakpoint",
        )
        self.assertEqual(
            len(lldbutil.continue_to_breakpoint(process, bkpt2)),
            1,
            "Hit second breakpoint",
        )



"""
Tests ThreadSanitizer's support to detect a leaked thread.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TsanThreadLeakTestCase(TestBase):
    @expectedFailureNetBSD
    @skipIfFreeBSD  # llvm.org/pr21136 runtimes not yet available by default
    @skipIfRemote
    @skipUnlessThreadSanitizer
    def test(self):
        self.build()
        self.tsan_tests()

    def tsan_tests(self):
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.runCmd("run")

        stop_reason = (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        )
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        # the stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            "A thread leak should be detected",
            substrs=["stopped", "stop reason = Thread leak detected"],
        )

        self.assertEqual(
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason(),
            lldb.eStopReasonInstrumentation,
        )



# RUN: %PYTHON %s | FileCheck %s

import gc
import sys
from mlir.ir import *
from mlir.dialects._ods_common import _cext


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testDialectDescriptor
@run
def testDialectDescriptor():
    ctx = Context()
    d = ctx.get_dialect_descriptor("func")
    # CHECK: <DialectDescriptor func>
    print(d)
    # CHECK: func
    print(d.namespace)
    try:
        _ = ctx.get_dialect_descriptor("not_existing")
    except ValueError:
        pass
    else:
        assert False, "Expected exception"


# CHECK-LABEL: TEST: testUserDialectClass
@run
def testUserDialectClass():
    ctx = Context()
    # Access using attribute.
    d = ctx.dialects.func
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)
    try:
        _ = ctx.dialects.not_existing
    except AttributeError:
        pass
    else:
        assert False, "Expected exception"

    # Access using index.
    d = ctx.dialects["func"]
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)
    try:
        _ = ctx.dialects["not_existing"]
    except IndexError:
        pass
    else:
        assert False, "Expected exception"

    # Using the 'd' alias.
    d = ctx.d["func"]
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)


# CHECK-LABEL: TEST: testCustomOpView
# This test uses the standard dialect AddFOp as an example of a user op.
# TODO: Op creation and access is still quite verbose: simplify this test as
# additional capabilities come online.
@run
def testCustomOpView():
    def createInput():
        op = Operation.create("pytest_dummy.intinput", results=[f32])
        # TODO: Auto result cast from operation
        return op.results[0]

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()

        with InsertionPoint(m.body):
            f32 = F32Type.get()
            # Create via dialects context collection.
            input1 = createInput()
            input2 = createInput()
            op1 = ctx.dialects.arith.AddFOp(input1, input2)

            # Create via an import
            from mlir.dialects.arith import AddFOp

            AddFOp(input1, op1.result)

    # CHECK: %[[INPUT0:.*]] = "pytest_dummy.intinput"
    # CHECK: %[[INPUT1:.*]] = "pytest_dummy.intinput"
    # CHECK: %[[R0:.*]] = arith.addf %[[INPUT0]], %[[INPUT1]] : f32
    # CHECK: %[[R1:.*]] = arith.addf %[[INPUT0]], %[[R0]] : f32
    m.operation.print()


# CHECK-LABEL: TEST: testIsRegisteredOperation
@run
def testIsRegisteredOperation():
    ctx = Context()

    # CHECK: cf.cond_br: True
    print(f"cf.cond_br: {ctx.is_registered_operation('cf.cond_br')}")
    # CHECK: func.not_existing: False
    print(f"func.not_existing: {ctx.is_registered_operation('func.not_existing')}")


# CHECK-LABEL: TEST: testAppendPrefixSearchPath
@run
def testAppendPrefixSearchPath():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        assert not _cext.globals._check_dialect_module_loaded("custom")
        Operation.create("custom.op")
        assert not _cext.globals._check_dialect_module_loaded("custom")

        sys.path.append(".")
        _cext.globals.append_dialect_search_prefix("custom_dialect")
        assert _cext.globals._check_dialect_module_loaded("custom")


# CHECK-LABEL: TEST: testDialectLoadOnCreate
@run
def testDialectLoadOnCreate():
    with Context(load_on_create_dialects=[]) as ctx:
        ctx.emit_error_diagnostics = True
        ctx.allow_unregistered_dialects = True

        def callback(d):
            # CHECK: DIAGNOSTIC
            # CHECK-SAME: op created with unregistered dialect
            print(f"DIAGNOSTIC={d.message}")
            return True

        handler = ctx.attach_diagnostic_handler(callback)
        loc = Location.unknown(ctx)
        try:
            op = Operation.create("arith.addi", loc=loc)
            ctx.allow_unregistered_dialects = False
            op.verify()
        except MLIRError as e:
            pass

    with Context(load_on_create_dialects=["func"]) as ctx:
        loc = Location.unknown(ctx)
        fn = Operation.create("func.func", loc=loc)

    # TODO: This may require an update if a site wide policy is set.
    # CHECK: Load on create: []
    print(f"Load on create: {get_load_on_create_dialects()}")
    append_load_on_create_dialect("func")
    # CHECK: Load on create:
    # CHECK-SAME: func
    print(f"Load on create: {get_load_on_create_dialects()}")
    print(get_load_on_create_dialects())



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import skipIfNetBSD, skipIf
import lldbdap_testcase


class TestDAP_launch_terminate_commands(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests that the "terminateCommands", that can be passed during
    launch, are run when the debugger is disconnected.
    """

    @skipIfNetBSD  # Hangs on NetBSD as well
    @skipIf(archs=["arm$", "aarch64"], oslist=["linux"])
    def test(self):
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")

        terminateCommands = ["expr 4+2"]
        self.launch(
            program,
            stopOnEntry=True,
            terminateCommands=terminateCommands,
            disconnectAutomatically=False,
        )
        self.get_console()
        # Once it's disconnected the console should contain the
        # "terminateCommands"
        self.dap_server.request_disconnect(terminateDebuggee=True)
        output = self.collect_console(pattern=terminateCommands[0])
        self.verify_commands("terminateCommands", output, terminateCommands)



"""
Test basic std::vector functionality but with a declaration from
the debug info (the Foo struct) as content.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDbgInfoContentVector(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler="clang", compiler_version=["<", "12.0"])
    @skipIf(macos_version=["<", "14.0"])
    @skipIf(
        bugnumber="ASTImport of lambdas not supported: https://github.com/llvm/llvm-project/issues/149477"
    )
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            vector_type = "std::vector<Foo>"
        else:
            vector_type = "std::vector<Foo, std::allocator<Foo> >"

        size_type = "size_type"
        value_type = "value_type"
        iterator = "iterator"
        # LLDB's formatter provides us with a artificial 'item' member.
        iterator_children = [ValueCheck(name="item")]
        riterator = "reverse_iterator"
        riterator_children = [
            ValueCheck(),  # Deprecated __t_ member; no need to check
            ValueCheck(name="current"),
        ]

        self.expect_expr(
            "a",
            result_type=vector_type,
            result_children=[
                ValueCheck(children=[ValueCheck(value="3")]),
                ValueCheck(children=[ValueCheck(value="1")]),
                ValueCheck(children=[ValueCheck(value="2")]),
            ],
        )

        self.expect_expr("a.size()", result_type=size_type, result_value="3")
        self.expect_expr("a.front().a", result_type="int", result_value="3")
        self.expect_expr("a[1].a", result_type="int", result_value="1")
        self.expect_expr("a.back().a", result_type="int", result_value="2")

        self.expect("expr std::reverse(a.begin(), a.end())")
        self.expect_expr("a.front().a", result_type="int", result_value="2")

        self.expect_expr("a.begin()->a", result_type="int", result_value="2")
        self.expect_expr("a.rbegin()->a", result_type="int", result_value="3")

        self.expect("expr a.pop_back()")
        self.expect_expr("a.back().a", result_type="int", result_value="1")
        self.expect_expr("a.size()", result_type=size_type, result_value="2")

        self.expect_expr("a.at(0).a", result_type="int", result_value="2")

        self.expect("expr a.push_back({4})")
        self.expect_expr("a.back().a", result_type="int", result_value="4")
        self.expect_expr("a.size()", result_type=size_type, result_value="3")

        self.expect_expr(
            "a.begin()", result_type=iterator, result_children=iterator_children
        )
        self.expect_expr(
            "a.rbegin()", result_type=riterator, result_children=riterator_children
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBitfieldIvars(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m")
        )

        self.expect_expr(
            "chb->hb->field1", result_type="unsigned int", result_value="0"
        )
        ## This should happen second
        self.expect_expr(
            "chb->hb->field2", result_type="unsigned int", result_value="1"
        )

        self.expect_expr("hb2->field1", result_type="unsigned int", result_value="10")
        self.expect_expr("hb2->field2", result_type="unsigned int", result_value="3")
        self.expect_expr("hb2->field3", result_type="unsigned int", result_value="4")

        self.expect(
            "frame var *hb2",
            substrs=["x =", "100", "field1 =", "10", "field2 =", "3", "field3 =", "4"],
        )

        self.expect_expr(
            "myField",
            result_type="UCBitFields",
            result_children=[
                ValueCheck(name="fieldOne", value="'\\0'"),
                ValueCheck(name="fieldTwo", value="'\\x01'"),
                ValueCheck(name="fieldThree", value="'\\0'"),
                ValueCheck(name="fieldFour", value="'\\0'"),
                ValueCheck(name="fieldFive", value="'\\x01'"),
            ],
        )

    # This test is meant to be xfailed, but running the test triggers an ASan
    # issue, so it must be skipped for now.
    @skip
    def testExprWholeObject(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m")
        )

        ## FIXME expression with individual bit-fields obtains correct values but not with the whole object
        self.expect(
            "expr *hb2",
            substrs=["x =", "100", "field1 =", "10", "field2 =", "3", "field3 =", "4"],
        )



"""
Test that breakpoints set on a bad address say they are bad.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class BadAddressBreakpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_bad_address_breakpoints(self):
        """Test that breakpoints set on a bad address say they are bad."""
        self.build()
        self.address_breakpoints()

    def address_breakpoints(self):
        """Test that breakpoints set on a bad address say they are bad."""
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.c")
        )

        # illegal_address will hold (optionally) an address that, if
        # used as a breakpoint, will generate an unresolved breakpoint.
        illegal_address = None

        # Walk through all the memory regions in the process and
        # find an address that is invalid.
        regions = process.GetMemoryRegions()
        for region_idx in range(regions.GetSize()):
            region = lldb.SBMemoryRegionInfo()
            regions.GetMemoryRegionAtIndex(region_idx, region)
            if illegal_address is None or region.GetRegionEnd() > illegal_address:
                illegal_address = region.GetRegionEnd()

        if illegal_address is not None:
            # Now, set a breakpoint at the address we know is illegal.
            bkpt = target.BreakpointCreateByAddress(illegal_address)
            # Verify that breakpoint is not resolved.
            for bp_loc in bkpt:
                self.assertFalse(bp_loc.IsResolved())
        else:
            self.fail(
                "Could not find an illegal address at which to set a bad breakpoint."
            )



"""
This is the llvm::Optional data formatter from llvm/utils/lldbDataFormatters.py
with the implementation replaced by bytecode.
"""

from __future__ import annotations
from formatter_bytecode import *
import lldb


def __lldb_init_module(debugger, internal_dict):
    debugger.HandleCommand("type category define --enabled llvm")

    debugger.HandleCommand(
        "type synthetic add -w llvm "
        f"-l {__name__}.MyOptionalSynthProvider "
        '-x "^MyOptional<.+>$"'
    )
    debugger.HandleCommand(
        "type summary add -w llvm "
        f"-e -F {__name__}.MyOptionalSummaryProvider "
        '-x "^MyOptional<.+>$"'
    )


def stringify(bytecode: bytearray) -> str:
    s = ""
    in_hex = False
    for b in bytecode:
        if (b < 32 or b > 127 or chr(b) in ['"', "`", "'"]) or (
            in_hex
            and chr(b).lower()
            in [
                "a",
                "b",
                "c",
                "d",
                "e",
                "f",
                "0",
                "1",
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
            ]
        ):
            s += r"\x" + hex(b)[2:]
            in_hex = True
        else:
            s += chr(b)
            in_hex = False
    return s


def evaluate(assembly: str, data: list):
    bytecode = assemble(assembly)
    trace = True
    if trace:
        print(
            "Compiled to {0} bytes of bytecode:\n{1}".format(
                len(bytecode), stringify(bytecode)
            )
        )
    result = interpret(bytecode, [], data, False)  # trace)
    if trace:
        print("--> {0}".format(result))
    return result


# def GetOptionalValue(valobj):
#    storage = valobj.GetChildMemberWithName("Storage")
#    if not storage:
#        storage = valobj
#
#    failure = 2
#    hasVal = storage.GetChildMemberWithName("hasVal").GetValueAsUnsigned(failure)
#    if hasVal == failure:
#        return "<could not read MyOptional>"
#
#    if hasVal == 0:
#        return None
#
#    underlying_type = storage.GetType().GetTemplateArgumentType(0)
#    storage = storage.GetChildMemberWithName("value")
#    return storage.Cast(underlying_type)


def MyOptionalSummaryProvider(valobj, internal_dict):
    #    val = GetOptionalValue(valobj)
    #    if val is None:
    #        return "None"
    #    if val.summary:
    #        return val.summary
    #    return val.GetValue()
    summary = ""
    summary += ' dup "Storage" @get_child_with_name call'  # valobj storage
    summary += " dup is_null ~ { swap } if drop"  # storage
    summary += ' dup "hasVal" @get_child_with_name call'  # storage obj(hasVal)
    summary += ' dup is_null { drop "<could not read MyOptional>" } {'
    summary += "   @get_value_as_unsigned call"  # storage int(hasVal)
    summary += '   0u = { "None" } {'
    summary += "     dup @get_type call"
    summary += "     0u @get_template_argument_type call"  # storage type
    summary += "     swap"  # type storage
    summary += '     "value" @get_child_with_name call'  # type value
    summary += "     swap @cast call"  # type(value)
    summary += '     dup is_null { "None" } {'
    summary += (
        "       dup @summary call dup @strlen call { @get_value call } { drop } ifelse"
    )
    summary += "     } ifelse"
    summary += "   } ifelse"
    summary += " } ifelse"
    return evaluate(summary, [valobj])


class MyOptionalSynthProvider:
    """Provides deref support to llvm::Optional<T>"""

    def __init__(self, valobj, internal_dict):
        self.valobj = valobj

    def num_children(self):
        # return self.valobj.num_children
        num_children = " @get_num_children call"
        return evaluate(num_children, [self.valobj])

    def get_child_index(self, name):
        # if name == "$$dereference$$":
        #    return self.valobj.num_children
        # return self.valobj.GetIndexOfChildWithName(name)
        get_child_index = ' dup "$$dereference$$" ='
        get_child_index += " { drop @get_num_children call } {"  # obj name
        get_child_index += "   @get_child_index call"  # index
        get_child_index += " } ifelse"
        return evaluate(get_child_index, [self.valobj, name])

    def get_child_at_index(self, index):
        # if index < self.valobj.num_children:
        #    return self.valobj.GetChildAtIndex(index)
        # return GetOptionalValue(self.valobj) or lldb.SBValue()
        get_child_at_index = " over over swap"  # obj index index obj
        get_child_at_index += " @get_num_children call"  # obj index index n
        get_child_at_index += " < { @get_child_at_index call } {"  # obj index

        get_opt_val = ' dup "Storage" @get_child_with_name call'  # valobj storage
        get_opt_val += " dup { swap } if drop"  # storage
        get_opt_val += ' dup "hasVal" @get_child_with_name call'  # storage
        get_opt_val += " @get_value_as_unsigned call"  # storage int(hasVal)
        get_opt_val += ' dup 2 = { drop "<could not read MyOptional>" } {'
        get_opt_val += '   0 = { "None" } {'
        get_opt_val += (
            "     dup @get_type call 0 @get_template_argument_type call"  # storage type
        )
        get_opt_val += "     swap"  # type storage
        get_opt_val += '     "value" @get_child_with_name call'  # type value
        get_opt_val += "     swap @cast call"  # type(value)
        get_opt_val += "   } ifelse"
        get_opt_val += " } ifelse"

        get_child_at_index += get_opt_val
        get_child_at_index += " } ifelse"

        return evaluate(get_child_at_index, [self.valobj, index])



"""
Test PHI nodes work in the IR interpreter.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class IRInterpreterPHINodesTestCase(TestBase):
    def test_phi_node_support(self):
        """Test support for PHI nodes in the IR interpreter."""

        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break on the first assignment to i
        line = line_number("main.cpp", "i = 5")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.runCmd("s")

        # The logical 'or' causes a PHI node to be generated. Execute without JIT
        # to test that the interpreter can handle this
        self.expect("expr -j 0 -- i == 3 || i == 5", substrs=["true"])

        self.runCmd("s")
        self.expect("expr -j 0 -- i == 3 || i == 5", substrs=["false"])
        self.runCmd("s")
        self.expect("expr -j 0 -- i == 3 || i == 5", substrs=["true"])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestProcessHandle(TestBase):
    @no_debug_info_test
    @skipIfWindows
    def test_process_handle(self):
        """Test that calling process handle before we have a target, and before we
        have a process will affect the process.  Also that the signal settings
        are preserved on rerun."""
        self.build()

        # Make sure we don't accept signal values by signo with no process - we don't know what the
        # mapping will be so we can't do the right thing with bare numbers:
        lldbutil.set_actions_for_signal(
            self, "9", "true", None, None, expect_success=False
        )

        # First, I need a reference value so I can see whether changes actually took:
        (target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        (default_pass, default_stop, default_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV"
        )

        # Let's change the value here, then exit and make sure the changed value sticks:
        new_value = "false"
        if default_pass == "true":
            new_value = "false"

        # First make sure we get an error for bogus values when running:
        lldbutil.set_actions_for_signal(
            self, "NOTSIGSEGV", new_value, None, None, expect_success=False
        )

        # Then set the one we intend to change.
        lldbutil.set_actions_for_signal(self, "SIGSEGV", new_value, None, None)

        process.Continue()

        self.assertState(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)

        # Check that we preserved the setting:
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV", from_target=True
        )
        self.assertEqual(curr_pass, new_value, "Pass was set correctly")
        self.assertEqual(curr_stop, "not set", "Stop was not set by us")
        self.assertEqual(curr_notify, "not set", "Notify was not set by us")

        # Run again and make sure that we prime the new process with these settings:
        process = lldbutil.run_to_breakpoint_do_run(self, target, bkpt)

        # We check the process settings now, to see what got copied into the process:
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV"
        )
        self.assertEqual(curr_pass, new_value, "Pass was set correctly")
        self.assertEqual(curr_stop, default_stop, "Stop was its default value")
        self.assertEqual(curr_notify, default_notify, "Notify was its default value")

        # Now kill this target, set the handling and make sure the values get copied from the dummy into the new target.
        success = self.dbg.DeleteTarget(target)
        self.assertTrue(success, "Deleted the target")
        self.assertEqual(self.dbg.GetNumTargets(), 0, "We did delete all the targets.")

        # The signal settings should be back at their default - we were only setting this on the target:
        lldbutil.get_actions_for_signal(
            self, "SIGSEGV", from_target=True, expected_absent=True
        )
        # Set a valid one:
        lldbutil.set_actions_for_signal(self, "SIGSEGV", new_value, None, None)
        # Set a bogus one - we don't have a way to check pre-run so this is allowed
        # but we should get an error message when launching:
        lldbutil.set_actions_for_signal(self, "SIGNOTSIG", new_value, None, None)

        out_filename = self.getBuildArtifact("output")
        success = True
        try:
            f = open(out_filename, "w")
        except:
            success = False

        if not success:
            self.fail("Couldn't open error output file for writing.")

        self.dbg.SetErrorFileHandle(f, False)
        # Now make a new process and make sure the right values got copied into the new target
        (target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        f.write("TESTPATTERN\n")
        f.flush()
        f.close()

        try:
            f = open(out_filename, "r")
        except:
            success = False

        if not success:
            self.fail("Couldn't open error output file for reading")
        errors = f.read()
        f.close()

        self.assertIn("SIGNOTSIG", errors, "We warned about the unset signal")
        # Also make sure we didn't accidentally add this bogus setting to the process.
        lldbutil.set_actions_for_signal(
            self, "SIGNOTSIG", "true", "true", "true", expect_success=False
        )

        # Check that they went into the target:
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV", from_target=True
        )
        self.assertEqual(curr_pass, new_value, "Pass was set correctly")
        self.assertEqual(curr_stop, "not set", "Stop was not set by us")
        self.assertEqual(curr_notify, "not set", "Notify was not set by us")

        # And the process:
        # Check that they went into the target:
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV"
        )
        self.assertEqual(curr_pass, new_value, "Pass was set correctly")
        self.assertEqual(curr_stop, default_stop, "Stop was its default value")
        self.assertEqual(curr_notify, default_notify, "Notify was its default value")

        # Now clear the handling, and make sure that we get the right signal values again:
        self.runCmd("process handle -c SIGSEGV")
        # Check that there is no longer configuration for SIGSEGV in the target:
        lldbutil.get_actions_for_signal(
            self, "SIGSEGV", from_target=True, expected_absent=True
        )
        # Make a new process, to make sure we did indeed reset the values:
        (target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV"
        )
        self.assertEqual(curr_pass, new_value, "Pass was set correctly")
        self.assertEqual(curr_stop, default_stop, "Stop was its default value")
        self.assertEqual(curr_notify, default_notify, "Notify was its default value")

        # Finally remove this from the dummy target as well, and make sure it was cleared from there:
        self.runCmd("process handle -c -d SIGSEGV")
        error = process.Kill()
        self.assertSuccess(error, "Killed the process")
        success = self.dbg.DeleteTarget(target)
        self.assertTrue(success, "Destroyed the target.")

        (target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        (curr_pass, curr_stop, curr_notify) = lldbutil.get_actions_for_signal(
            self, "SIGSEGV"
        )
        self.assertEqual(curr_pass, default_pass, "Pass was set correctly")
        self.assertEqual(curr_stop, default_stop, "Stop was its default value")
        self.assertEqual(curr_notify, default_notify, "Notify was its default value")



"""
Test that we can backtrace up an ARM Cortex-M Exception return stack
"""

import lldb
import json
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCortexMExceptionUnwind(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfLLVMTargetMissing("ARM")
    def test_no_fpu(self):
        """Test that we can backtrace correctly through an ARM Cortex-M Exception return stack"""

        target = self.dbg.CreateTarget("")
        exe = "binary.json"
        with open(exe) as f:
            exe_json = json.load(f)
            exe_uuid = exe_json["uuid"]

        target.AddModule(exe, "", exe_uuid)
        self.assertTrue(target.IsValid())

        core = self.getBuildArtifact("core")
        self.yaml2macho_core("armv7m-nofpu-exception.yaml", core, exe_uuid)

        if self.TraceOn():
            self.runCmd("log enable lldb unwind")

        process = target.LoadCore(core)
        self.assertTrue(process.IsValid())

        if self.TraceOn():
            self.runCmd("target list")
            self.runCmd("image list")
            self.runCmd("target modules dump sections")
            self.runCmd("target modules dump symtab")
            self.runCmd("bt")

        thread = process.GetThreadAtIndex(0)
        self.assertTrue(thread.IsValid())

        self.assertEqual(thread.GetNumFrames(), 3)
        stackframe_names = [
            "exception_catcher",
            "exception_thrower",
            "main",
        ]
        for i, name in enumerate(stackframe_names):
            self.assertEqual(name, thread.GetFrameAtIndex(i).GetSymbol().GetName())



# RUN: %PYTHON %s
# It is sufficient that this doesn't assert.

from mlir.ir import *


def createDetachedModule():
    module = Module.create()
    with InsertionPoint(module.body):
        # TODO: Python bindings are currently unaware that modules are also
        # operations, so having a module erased won't trigger the cascading
        # removal of live operations (#93337). Use a non-module operation
        # instead.
        nested = Operation.create("test.some_operation", regions=1)

        # When the operation is detached from parent, it is considered to be
        # owned by Python. It will therefore be erased when the Python object
        # is destroyed.
        nested.detach_from_parent()

        # However, we create and maintain references to operations within
        # `nested`. These references keep the corresponding operations in the
        # "live" list even if they have been erased in C++, making them
        # "zombie". If the C++ allocator reuses one of the address previously
        # used for a now-"zombie" operation, this used to result in an
        # assertion "cannot create detached operation that already exists" from
        # the bindings code. Erasing the detached operation should result in
        # removing all nested operations from the live list.
        #
        # Note that the assertion is not guaranteed since it depends on the
        # behavior of the allocator on the C++ side, so this test mail fail
        # intermittently.
        with InsertionPoint(nested.regions[0].blocks.append()):
            a = [Operation.create("test.some_other_operation") for i in range(100)]
    return a


def createManyDetachedModules():
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        for j in range(100):
            a = createDetachedModule()


if __name__ == "__main__":
    createManyDetachedModules()



import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestTemplateAlias(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self, extra_flags):
        self.build(dictionary=extra_flags)
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        lldbutil.run_to_source_breakpoint(self, "return", lldb.SBFileSpec("main.cpp"))

        self.expect_expr("f1", result_type="Foo<int>")
        self.expect_expr("f2", result_type="Foo<double>")
        self.expect_expr("b1", result_type="Bar<int>")
        self.expect_expr("b2", result_type="Bar<double>")
        self.expect_expr("bf1", result_type="Bar<int>")
        self.expect_expr("bf2", result_type="Bar<double>")
        self.expect_expr("bf1", result_type="Bar<int>")
        self.expect_expr("bf2", result_type="Bar<double>")
        self.expect_expr("cbf1", result_type="Container<int>")

    @skipIf(compiler="clang", compiler_version=["<", "21"])
    @expectedFailureAll(
        bugnumber="LLDB doesn't reconstruct template alias names from template parameters"
    )
    def test_tag_alias_simple(self):
        self.do_test(
            dict(CXXFLAGS_EXTRAS="-gdwarf-5 -gtemplate-alias -gsimple-template-names")
        )

    @skipIf(compiler="clang", compiler_version=["<", "21"])
    def test_tag_alias_no_simple(self):
        self.do_test(
            dict(
                CXXFLAGS_EXTRAS="-gdwarf-5 -gtemplate-alias -gno-simple-template-names"
            )
        )

    @skipIf(compiler="clang", compiler_version=["<", "21"])
    def test_no_tag_alias_simple(self):
        self.do_test(
            dict(
                CXXFLAGS_EXTRAS="-gdwarf-5 -gno-template-alias -gsimple-template-names"
            )
        )

    @skipIf(compiler="clang", compiler_version=["<", "21"])
    def test_no_tag_alias_no_simple(self):
        self.do_test(
            dict(
                CXXFLAGS_EXTRAS="-gdwarf-5 -gno-template-alias -gno-simple-template-names"
            )
        )



import lldb
import os
import re
import socket
import time

from contextlib import closing
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest
from lldbgdbserverutils import get_lldb_server_exe


# PExpect uses many timeouts internally and doesn't play well
# under ASAN on a loaded machine..
@skipIfAsan
class TestStatusline(PExpectTest):
    # Change this value to something smaller to make debugging this test less
    # tedious.
    TIMEOUT = 60

    TERMINAL_HEIGHT = 10
    TERMINAL_WIDTH = 60

    def do_setup(self):
        # Create a target and run to a breakpoint.
        exe = self.getBuildArtifact("a.out")
        self.expect(
            "target create {}".format(exe), substrs=["Current executable set to"]
        )
        self.expect('breakpoint set -p "Break here"', substrs=["Breakpoint 1"])
        self.expect("run", substrs=["stop reason"])
        self.resize()

    def resize(self, height=None, width=None):
        height = self.TERMINAL_HEIGHT if not height else height
        width = self.TERMINAL_WIDTH if not width else width
        # Change the terminal dimensions. When we launch the tests, we reset
        # all the settings, leaving the terminal dimensions unset.
        self.child.setwinsize(height, width)

    def test(self):
        """Basic test for the statusline."""
        self.build()
        self.launch(timeout=self.TIMEOUT)
        self.do_setup()

        # Enable the statusline and check for the control character and that we
        # can see the target, the location and the stop reason.
        self.expect('set set separator "| "')
        self.expect(
            "set set show-statusline true",
            [
                "\x1b[1;{}r".format(self.TERMINAL_HEIGHT - 1),
                "a.out | main.c:2:11 | breakpoint 1.1                        ",
            ],
        )

        # Change the terminal dimensions and make sure it's reflected immediately.
        self.child.setwinsize(self.TERMINAL_HEIGHT, 25)
        self.child.expect(re.escape("a.out | main.c:2:11 | bre"))
        self.child.setwinsize(self.TERMINAL_HEIGHT, self.TERMINAL_WIDTH)

        # Change the separator.
        self.expect('set set separator "S "', ["a.out S main.c:2:11"])

        # Change the format.
        self.expect(
            'set set statusline-format "target = {${target.file.basename}} ${separator}"',
            ["target = a.out S"],
        )
        self.expect('set set separator "| "')

        # Hide the statusline and check for the control character.
        self.expect(
            "set set show-statusline false", ["\x1b[1;{}r".format(self.TERMINAL_HEIGHT)]
        )

    def test_no_color(self):
        """Basic test for the statusline with colors disabled."""
        self.build()
        self.launch(use_colors=False, timeout=self.TIMEOUT)
        self.do_setup()

        # Enable the statusline and check for the "reverse video" control character.
        self.expect(
            "set set show-statusline true",
            [
                "\x1b[7m",
            ],
        )

    def test_deadlock(self):
        """Regression test for lock inversion between the statusline mutex and
        the output mutex."""
        self.build()
        self.launch(
            extra_args=["-o", "settings set use-color false"], timeout=self.TIMEOUT
        )
        self.child.expect("(lldb)")
        self.resize()

        exe = self.getBuildArtifact("a.out")

        self.expect("file {}".format(exe), ["Current executable"])
        self.expect("help", ["Debugger commands"])

    def test_no_target(self):
        """Test that we print "no target" when launched without a target."""
        self.launch(timeout=self.TIMEOUT)
        self.resize()

        self.expect("set set show-statusline true", ["no target"])

    @skipIfEditlineSupportMissing
    def test_resize(self):
        """Test that move the cursor when resizing."""
        self.launch(timeout=self.TIMEOUT)
        self.resize()
        self.expect("set set show-statusline true", ["no target"])
        self.resize(20, 60)
        # Check for the escape code to resize the scroll window.
        self.child.expect(re.escape("\x1b[1;19r"))
        self.child.expect("(lldb)")

    @skipIfRemote
    @skipIfWindows
    @skipIfDarwin
    @skipIfLinux # https://github.com/llvm/llvm-project/issues/154763
    @add_test_categories(["lldb-server"])
    def test_modulelist_deadlock(self):
        """Regression test for a deadlock that occurs when the status line is enabled before connecting
        to a gdb-remote server.
        """
        if get_lldb_server_exe() is None:
            self.skipTest("lldb-server not found")

        MAX_RETRY_ATTEMPTS = 10
        DELAY = 0.25

        def _find_free_port(host):
            for attempt in range(MAX_RETRY_ATTEMPTS):
                try:
                    family, type, proto, _, _ = socket.getaddrinfo(
                        host, 0, proto=socket.IPPROTO_TCP
                    )[0]
                    with closing(socket.socket(family, type, proto)) as sock:
                        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                        sock.bind((host, 0))
                        return sock.getsockname()
                except OSError:
                    time.sleep(DELAY * 2**attempt)  # Exponential backoff
            raise RuntimeError(
                "Could not find a free port on {} after {} attempts.".format(
                    host, MAX_RETRY_ATTEMPTS
                )
            )

        def _wait_for_server_ready_in_log(log_file_path, ready_message):
            """Check log file for server ready message with retry logic"""
            for attempt in range(MAX_RETRY_ATTEMPTS):
                if os.path.exists(log_file_path):
                    with open(log_file_path, "r") as f:
                        if ready_message in f.read():
                            return
                time.sleep(pow(2, attempt) * DELAY)
            raise RuntimeError(
                "Server not ready after {} attempts.".format(MAX_RETRY_ATTEMPTS)
            )

        self.build()
        exe_path = self.getBuildArtifact("a.out")
        server_log_file = self.getLogBasenameForCurrentTest() + "-lldbserver.log"
        self.addTearDownHook(
            lambda: os.path.exists(server_log_file) and os.remove(server_log_file)
        )

        # Find a free port for the server
        addr = _find_free_port("localhost")
        connect_address = "[{}]:{}".format(*addr)
        commandline_args = [
            "gdbserver",
            connect_address,
            exe_path,
            "--log-file={}".format(server_log_file),
            "--log-channels=lldb conn",
        ]

        server_proc = self.spawnSubprocess(
            get_lldb_server_exe(), commandline_args, install_remote=False
        )
        self.assertIsNotNone(server_proc)

        # Wait for server to be ready by checking log file.
        server_ready_message = "Listen to {}".format(connect_address)
        _wait_for_server_ready_in_log(server_log_file, server_ready_message)

        # Launch LLDB client and connect to lldb-server with statusline enabled
        self.launch(timeout=self.TIMEOUT)
        self.resize()
        self.expect("settings set show-statusline true", ["no target"])
        self.expect(
            f"gdb-remote {connect_address}", [b"a.out \xe2\x94\x82 signal SIGSTOP"]
        )



""" Testing separate debug info loading for base binary with a symlink. """
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestTargetSymbolsSepDebugSymlink(TestBase):
    @no_debug_info_test  # Prevent the genaration of the dwarf version of this test
    @skipUnlessPlatform(["linux"])
    @skipIf(hostoslist=["windows"])
    @skipIfRemote  # llvm.org/pr36237
    def test_target_symbols_sepdebug_symlink_case(self):
        self.build()
        exe = self.getBuildArtifact("dirsymlink/stripped.symlink")

        lldbutil.run_to_name_breakpoint(self, "main", exe_name=exe)



#!/usr/bin/env python

import argparse
import itertools
import os
import re
import sys
from collections import defaultdict

from use_lldb_suite import lldb_root

parser = argparse.ArgumentParser(
    description="Analyze LLDB project #include dependencies."
)
parser.add_argument(
    "--show-counts",
    default=False,
    action="store_true",
    help="When true, show the number of dependencies from each subproject",
)
parser.add_argument(
    "--discover-cycles",
    default=False,
    action="store_true",
    help="When true, find and display all project dependency cycles.  Note,"
    "this option is very slow",
)

args = parser.parse_args()

src_dir = os.path.join(lldb_root, "source")
inc_dir = os.path.join(lldb_root, "include")

src_map = {}

include_regex = re.compile('#include "((lldb|Plugins|clang)(.*/)+).*"')


def is_sublist(small, big):
    it = iter(big)
    return all(c in it for c in small)


def normalize_host(str):
    if str.startswith("lldb/Host"):
        return "lldb/Host"
    if str.startswith("Plugins"):
        return "lldb/" + str
    if str.startswith("lldb/../../source"):
        return str.replace("lldb/../../source", "lldb")
    return str


def scan_deps(this_dir, file):
    global src_map
    deps = {}
    this_dir = normalize_host(this_dir)
    if this_dir in src_map:
        deps = src_map[this_dir]

    with open(file) as f:
        for line in list(f):
            m = include_regex.match(line)
            if m is None:
                continue
            relative = m.groups()[0].rstrip("/")
            if relative == this_dir:
                continue
            relative = normalize_host(relative)
            if relative in deps:
                deps[relative] += 1
            elif relative != this_dir:
                deps[relative] = 1
    if this_dir not in src_map and len(deps) > 0:
        src_map[this_dir] = deps


for base, dirs, files in os.walk(inc_dir):
    dir = os.path.basename(base)
    relative = os.path.relpath(base, inc_dir)
    inc_files = [x for x in files if os.path.splitext(x)[1] in [".h"]]
    relative = relative.replace("\\", "/")
    for inc in inc_files:
        inc_path = os.path.join(base, inc)
        scan_deps(relative, inc_path)

for base, dirs, files in os.walk(src_dir):
    dir = os.path.basename(base)
    relative = os.path.relpath(base, src_dir)
    src_files = [x for x in files if os.path.splitext(x)[1] in [".cpp", ".h", ".mm"]]
    norm_base_path = os.path.normpath(os.path.join("lldb", relative))
    norm_base_path = norm_base_path.replace("\\", "/")
    for src in src_files:
        src_path = os.path.join(base, src)
        scan_deps(norm_base_path, src_path)
    pass


def is_existing_cycle(path, cycles):
    # If we have a cycle like # A -> B -> C (with an implicit -> A at the end)
    # then we don't just want to check for an occurrence of A -> B -> C in the
    # list of known cycles, but every possible rotation of A -> B -> C.  For
    # example, if we previously encountered B -> C -> A (with an implicit -> B
    # at the end), then A -> B -> C is also a cycle.  This is an important
    # optimization which reduces the search space by multiple orders of
    # magnitude.
    for i in range(0, len(path)):
        if any(is_sublist(x, path) for x in cycles):
            return True
        path = [path[-1]] + path[0:-1]
    return False


def expand(path_queue, path_lengths, cycles, src_map):
    # We do a breadth first search, to make sure we visit all paths in order
    # of ascending length.  This is an important optimization to make sure that
    # short cycles are discovered first, which will allow us to discard longer
    # cycles which grow the search space exponentially the longer they get.
    while len(path_queue) > 0:
        cur_path = path_queue.pop(0)
        if is_existing_cycle(cur_path, cycles):
            continue

        next_len = path_lengths.pop(0) + 1
        last_component = cur_path[-1]

        for item in src_map.get(last_component, []):
            if item.startswith("clang"):
                continue

            if item in cur_path:
                # This is a cycle.  Minimize it and then check if the result is
                # already in the list of cycles.  Insert it (or not) and then
                # exit.
                new_index = cur_path.index(item)
                cycle = cur_path[new_index:]
                if not is_existing_cycle(cycle, cycles):
                    cycles.append(cycle)
                continue

            path_lengths.append(next_len)
            path_queue.append(cur_path + [item])
    pass


cycles = []

path_queue = [[x] for x in iter(src_map)]
path_lens = [1] * len(path_queue)

items = list(src_map.items())
items.sort(key=lambda A: A[0])

for path, deps in items:
    print(path + ":")
    sorted_deps = list(deps.items())
    if args.show_counts:
        sorted_deps.sort(key=lambda A: (A[1], A[0]))
        for dep in sorted_deps:
            print("\t{} [{}]".format(dep[0], dep[1]))
    else:
        sorted_deps.sort(key=lambda A: A[0])
        for dep in sorted_deps:
            print("\t{}".format(dep[0]))


def iter_cycles(cycles):
    global src_map
    for cycle in cycles:
        cycle.append(cycle[0])
        zipper = list(zip(cycle[0:-1], cycle[1:]))
        result = [(x, src_map[x][y], y) for (x, y) in zipper]
        total = 0
        smallest = result[0][1]
        for first, value, last in result:
            total += value
            smallest = min(smallest, value)
        yield (total, smallest, result)


if args.discover_cycles:
    print("Analyzing cycles...")

    expand(path_queue, path_lens, cycles, src_map)

    average = sum([len(x) + 1 for x in cycles]) / len(cycles)

    print("Found {} cycles.  Average cycle length = {}.".format(len(cycles), average))
    counted = list(iter_cycles(cycles))
    if args.show_counts:
        counted.sort(key=lambda A: A[0])
        for total, smallest, cycle in counted:
            sys.stdout.write("{} deps to break: ".format(total))
            sys.stdout.write(cycle[0][0])
            for first, count, last in cycle:
                sys.stdout.write(" [{}->] {}".format(count, last))
            sys.stdout.write("\n")
    else:
        for cycle in cycles:
            cycle.append(cycle[0])
            print(" -> ".join(cycle))

    print("Analyzing islands...")
    islands = []
    outgoing_counts = defaultdict(int)
    incoming_counts = defaultdict(int)
    for total, smallest, cycle in counted:
        for first, count, last in cycle:
            outgoing_counts[first] += count
            incoming_counts[last] += count
    for cycle in cycles:
        this_cycle = set(cycle)
        disjoints = [x for x in islands if this_cycle.isdisjoint(x)]
        overlaps = [x for x in islands if not this_cycle.isdisjoint(x)]
        islands = disjoints + [set.union(this_cycle, *overlaps)]
    print("Found {} disjoint cycle islands...".format(len(islands)))
    for island in islands:
        print("Island ({} elements)".format(len(island)))
        sorted = []
        for node in island:
            sorted.append((node, incoming_counts[node], outgoing_counts[node]))
        sorted.sort(key=lambda x: x[1] + x[2])
        for node, inc, outg in sorted:
            print("  {} [{} in, {} out]".format(node, inc, outg))
    sys.stdout.flush()
pass



import lldb


class myArraySynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj

    def num_children(self):
        size_valobj = self.valobj.GetChildMemberWithName("m_arr_size")
        if size_valobj:
            return size_valobj.GetValueAsUnsigned(0)
        return 0

    def get_child_at_index(self, index):
        size_valobj = self.valobj.GetChildMemberWithName("m_arr_size")
        arr = self.valobj.GetChildMemberWithName("m_array")
        if not size_valobj or not arr:
            return None
        max_idx = size_valobj.GetValueAsUnsigned(0)
        if index >= max_idx:
            return None
        return arr.GetChildAtIndex(index)

    def get_child_index(self, name):
        if name == "[0]":
            return 0
        if name == "[1]":
            return 1
        if name == "[2]":
            return 2
        if name == "[3]":
            return 3
        return -1



import os
import shutil
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


def has_lldb_codesign():
    """Check if the lldb_codesign certificate is available."""
    try:
        result = subprocess.run(
            [
                "security",
                "find-certificate",
                "-c",
                "lldb_codesign",
                "/Library/Keychains/System.keychain",
            ],
            capture_output=True,
        )
        return result.returncode == 0
    except FileNotFoundError:
        return False


@skipUnlessDarwin
class TestdSYMCodesign(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def build_dsym_with_script(self):
        self.build(debug_info="dsym")
        exe = self.getBuildArtifact("a.out")
        dsym = self.getBuildArtifact("a.out.dSYM")
        python_dir = os.path.join(dsym, "Contents", "Resources", "Python")
        os.makedirs(python_dir, exist_ok=True)
        shutil.copy(
            os.path.join(self.getSourceDir(), "dsym_script.py"),
            os.path.join(python_dir, "a.py"),
        )
        return exe, dsym

    def test_adhoc_signed_dsym(self):
        """An ad-hoc signed dSYM should not be loaded because the
        signature doesn't chain to a trusted root CA."""
        exe, dsym = self.build_dsym_with_script()
        subprocess.check_call(["codesign", "-f", "-s", "-", dsym])

        self.runCmd("settings set target.load-script-from-symbol-file trusted")
        self.createTestTarget(file_path=exe)

        self.expect(
            "script -- print('SENTINEL')",
            substrs=["SENTINEL"],
        )
        # The script should NOT have been loaded.
        self.assertFalse(
            hasattr(lldb, "_dsym_codesign_test_loaded"),
            "Script should not auto-load from ad-hoc signed dSYM",
        )

    @unittest.skipUnless(has_lldb_codesign(), "requires lldb_codesign certificate")
    def test_trusted_signed_dsym_auto_loads(self):
        """A dSYM signed with the trusted lldb_codesign certificate should
        auto-load scripts."""
        exe, dsym = self.build_dsym_with_script()
        subprocess.check_call(["codesign", "-f", "-s", "lldb_codesign", dsym])

        self.runCmd("settings set target.load-script-from-symbol-file trusted")
        self.createTestTarget(file_path=exe)

        # The script sets a marker attribute on the lldb module.
        self.assertTrue(
            getattr(lldb, "_dsym_codesign_test_loaded", False),
            "Script should auto-load from trusted signed dSYM",
        )



"""This file contains functions related to interpreting measurement results
of benchmarks.
"""
import configparser
import numpy as np
import os


def has_enough_measurements(measurements):
    """Takes a list/numpy array of measurements and determines whether we have
    enough measurements to make a confident judgement of the performance. The
    criteria for determining whether we have enough measurements is as follows.
    1. Whether enough time, defaulting to 1 second, has passed.
    2. Whether we have a max number of measurements, defaulting to a billion.

    If 1. is true, 2. doesn't need to be true.
    """
    config = configparser.ConfigParser()
    config.read(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.cfg"))
    if "stats" in config:
        stats_dict = {
            "max_number_of_measurements": int(
                float(config["stats"]["max_number_of_measurements"])
            ),
            "max_time_for_a_benchmark_ns": int(
                float(config["stats"]["max_time_for_a_benchmark_ns"])
            ),
        }
    else:
        stats_dict = {
            "max_number_of_measurements": 1e9,
            "max_time_for_a_benchmark_ns": 1e9,
        }
    return (
        np.sum(measurements) >= stats_dict["max_time_for_a_benchmark_ns"]
        or np.size(measurements) >= stats_dict["max_number_of_measurements"]
    )



"""
Test some lldb command abbreviations.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DisassemblyTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.build()
        target, _, _, bkpt = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )
        self.runCmd("dis -f")
        disassembly_with_break = self.res.GetOutput().splitlines()

        self.assertTrue(target.BreakpointDelete(bkpt.GetID()))

        self.runCmd("dis -f")
        disassembly_without_break = self.res.GetOutput().splitlines()

        # Make sure all assembly instructions are the same as instructions
        # with the breakpoint removed.
        self.assertEqual(len(disassembly_with_break), len(disassembly_without_break))
        for dis_inst_with, dis_inst_without in zip(
            disassembly_with_break, disassembly_without_break
        ):
            inst_with = dis_inst_with.split(":")[-1]
            inst_without = dis_inst_without.split(":")[-1]
            self.assertEqual(inst_with, inst_without)



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class DataFormatterHexCapsTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format delete hex", check=False)
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("type format add -f uppercase int")

        self.expect(
            "frame variable mine",
            substrs=["mine = ", "first = 0x001122AA", "second = 0x1122BB44"],
        )

        self.runCmd("type format add -f hex int")

        self.expect(
            "frame variable mine",
            substrs=["mine = ", "first = 0x001122aa", "second = 0x1122bb44"],
        )

        self.runCmd("type format delete int")

        self.runCmd('type summary add -s "${var.first%X} and ${var.second%x}" foo')

        self.expect(
            "frame variable mine", substrs=["(foo) mine = 0x001122AA and 0x1122bb44"]
        )

        self.runCmd('type summary add -s "${var.first%X} and ${var.second%X}" foo')
        self.runCmd("next")
        self.runCmd("next")
        self.expect(
            "frame variable mine", substrs=["(foo) mine = 0xAABBCCDD and 0x1122BB44"]
        )

        self.runCmd('type summary add -s "${var.first%x} and ${var.second%X}" foo')
        self.expect(
            "frame variable mine", substrs=["(foo) mine = 0xaabbccdd and 0x1122BB44"]
        )
        self.runCmd("next")
        self.runCmd("next")
        self.runCmd('type summary add -s "${var.first%x} and ${var.second%x}" foo')
        self.expect(
            "frame variable mine", substrs=["(foo) mine = 0xaabbccdd and 0xff00ff00"]
        )
        self.runCmd('type summary add -s "${var.first%X} and ${var.second%X}" foo')
        self.expect(
            "frame variable mine", substrs=["(foo) mine = 0xAABBCCDD and 0xFF00FF00"]
        )



"""
Test the functionality of interactive scripted processes
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import json, os


class TestInteractiveScriptedProcess(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Build and load test program
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.script_module = "interactive_scripted_process"
        self.script_file = self.script_module + ".py"

    # These tests are flakey and sometimes timeout.  They work most of the time
    # so the basic event flow is right, but somehow the handling is off.
    @skipUnlessDarwin
    @skipIfDarwin
    def test_passthrough_launch(self):
        """Test a simple pass-through process launch"""
        self.passthrough_launch()

        lldbutil.run_break_set_by_source_regexp(self, "also break here")
        self.assertEqual(self.mux_target.GetNumBreakpoints(), 2)
        error = self.mux_process.Continue()
        self.assertSuccess(error, "Resuming multiplexer scripted process")
        self.assertTrue(self.mux_process.IsValid(), "Got a valid process")

        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), self.mux_process.GetBroadcaster(), timeout=20
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)
        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

    @skipUnlessDarwin
    @skipIfDarwin
    def test_multiplexed_launch(self):
        """Test a multiple interactive scripted process debugging"""
        self.passthrough_launch()
        self.assertEqual(self.dbg.GetNumTargets(), 2)

        driving_target = self.mux_process.GetScriptedImplementation().driving_target
        self.assertTrue(driving_target.IsValid(), "Driving target is invalid")

        # Create a target for the multiplexed even scripted process
        even_target = self.duplicate_target(driving_target)
        self.assertTrue(
            even_target.IsValid(),
            "Couldn't duplicate driving target to launch multiplexed even scripted process",
        )

        class_name = f"{self.script_module}.MultiplexedScriptedProcess"
        dictionary = {"driving_target_idx": self.dbg.GetIndexOfTarget(self.mux_target)}

        dictionary["parity"] = 0
        muxed_launch_info = self.get_launch_info(class_name, dictionary)

        # Launch Even Child Scripted Process
        error = lldb.SBError()
        even_process = even_target.Launch(muxed_launch_info, error)
        self.assertTrue(
            even_process, "Couldn't launch multiplexed even scripted process"
        )
        self.multiplex(even_process)

        # Check that the even process started running
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), even_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)
        # Check that the even process stopped
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), even_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

        self.assertTrue(even_process.IsValid(), "Got a valid process")
        self.assertState(
            even_process.GetState(), lldb.eStateStopped, "Process is stopped"
        )

        # Create a target for the multiplexed odd scripted process
        odd_target = self.duplicate_target(driving_target)
        self.assertTrue(
            odd_target.IsValid(),
            "Couldn't duplicate driving target to launch multiplexed odd scripted process",
        )

        dictionary["parity"] = 1
        muxed_launch_info = self.get_launch_info(class_name, dictionary)

        # Launch Odd Child Scripted Process
        error = lldb.SBError()
        odd_process = odd_target.Launch(muxed_launch_info, error)
        self.assertTrue(odd_process, "Couldn't launch multiplexed odd scripted process")
        self.multiplex(odd_process)

        # Check that the odd process started running
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), odd_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)
        # Check that the odd process stopped
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), odd_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

        self.assertTrue(odd_process.IsValid(), "Got a valid process")
        self.assertState(
            odd_process.GetState(), lldb.eStateStopped, "Process is stopped"
        )

        # Set a breakpoint on the odd child process
        bkpt = odd_target.BreakpointCreateBySourceRegex(
            "also break here", self.main_source_file
        )
        self.assertEqual(odd_target.GetNumBreakpoints(), 1)
        self.assertTrue(bkpt, "Second breakpoint set on child scripted process")
        self.assertEqual(bkpt.GetNumLocations(), 1, "Second breakpoint has 1 location")

        # Verify that the breakpoint was also set on the multiplexer & real target
        self.assertEqual(self.mux_target.GetNumBreakpoints(), 2)
        bkpt = self.mux_target.GetBreakpointAtIndex(1)
        self.assertEqual(
            bkpt.GetNumLocations(), 1, "Second breakpoint set on mux scripted process"
        )
        self.assertTrue(bkpt.MatchesName("multiplexed_scripted_process_421"))

        self.assertGreater(driving_target.GetNumBreakpoints(), 1)

        # Resume execution on child process
        error = odd_process.Continue()
        self.assertSuccess(error, "Resuming odd child scripted process")
        self.assertTrue(odd_process.IsValid(), "Got a valid process")

        # Since all the execution is asynchronous, the order in which events
        # arrive is non-deterministic, so we need a data structure to make sure
        # we received both the running and stopped event for each target.

        # Initialize the execution event "bingo book", that maps a process index
        # to a dictionary that contains flags that are not set for the process
        # events that we care about (running & stopped)

        execution_events = {
            1: {lldb.eStateRunning: False, lldb.eStateStopped: False},
            2: {lldb.eStateRunning: False, lldb.eStateStopped: False},
            3: {lldb.eStateRunning: False, lldb.eStateStopped: False},
        }

        def fetch_process_event(self, execution_events):
            event = lldbutil.fetch_next_event(
                self,
                self.dbg.GetListener(),
                lldb.SBProcess.GetBroadcasterClass(),
                match_class=True,
            )
            state = lldb.SBProcess.GetStateFromEvent(event)
            self.assertIn(state, [lldb.eStateRunning, lldb.eStateStopped])
            event_process = lldb.SBProcess.GetProcessFromEvent(event)
            self.assertTrue(event_process.IsValid())
            event_target = event_process.GetTarget()
            event_target_idx = self.dbg.GetIndexOfTarget(event_target)
            self.assertFalse(
                execution_events[event_target_idx][state],
                "Event already received for this process",
            )
            execution_events[event_target_idx][state] = True

        for _ in range((self.dbg.GetNumTargets() - 1) * 2):
            fetch_process_event(self, execution_events)

        for target_index, event_states in execution_events.items():
            for state, is_set in event_states.items():
                self.assertTrue(is_set, f"Target {target_index} has state {state} set")

        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)

        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

    def duplicate_target(self, driving_target):
        exe = driving_target.executable.fullpath
        triple = driving_target.triple
        return self.dbg.CreateTargetWithFileAndTargetTriple(exe, triple)

    def get_launch_info(self, class_name, script_dict):
        structured_data = lldb.SBStructuredData()
        structured_data.SetFromJSON(json.dumps(script_dict))

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName(class_name)
        launch_info.SetScriptedProcessDictionary(structured_data)
        return launch_info

    def multiplex(self, muxed_process):
        muxed_process.GetScriptedImplementation().multiplexer = (
            self.mux_process.GetScriptedImplementation()
        )
        self.mux_process.GetScriptedImplementation().multiplexed_processes[
            muxed_process.GetProcessID()
        ] = muxed_process

    def passthrough_launch(self):
        """Test that a simple passthrough wrapper functions correctly"""
        # First build the real target:
        self.assertEqual(self.dbg.GetNumTargets(), 1)
        real_target_id = 0
        real_target = self.dbg.GetTargetAtIndex(real_target_id)
        lldbutil.run_break_set_by_source_regexp(self, "Break here")
        self.assertEqual(real_target.GetNumBreakpoints(), 1)

        # Now source in the scripted module:
        script_path = os.path.join(self.getSourceDir(), self.script_file)
        self.runCmd(f"command script import '{script_path}'")

        self.mux_target = self.duplicate_target(real_target)
        self.assertTrue(self.mux_target.IsValid(), "duplicate target succeeded")

        mux_class = f"{self.script_module}.MultiplexerScriptedProcess"
        script_dict = {"driving_target_idx": real_target_id}
        mux_launch_info = self.get_launch_info(mux_class, script_dict)
        self.mux_process_listener = lldb.SBListener(
            "lldb.test.interactive-scripted-process.listener"
        )
        mux_launch_info.SetShadowListener(self.mux_process_listener)

        self.dbg.SetAsync(True)
        error = lldb.SBError()
        self.mux_process = self.mux_target.Launch(mux_launch_info, error)
        self.assertSuccess(error, "Launched multiplexer scripted process")
        self.assertTrue(self.mux_process.IsValid(), "Got a valid process")

        # Check that the real process started running
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)
        # Check that the mux process started running
        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateRunning)

        # Check that the real process stopped
        event = lldbutil.fetch_next_event(
            self, self.dbg.GetListener(), self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)
        # Check that the mux process stopped
        event = lldbutil.fetch_next_event(
            self, self.mux_process_listener, self.mux_process.GetBroadcaster()
        )
        self.assertState(lldb.SBProcess.GetStateFromEvent(event), lldb.eStateStopped)

        real_process = real_target.GetProcess()
        self.assertTrue(real_process.IsValid(), "Got a valid process")
        self.assertState(
            real_process.GetState(), lldb.eStateStopped, "Process is stopped"
        )

        # This is a passthrough, so the two processes should have the same state:
        # Check that we got the right threads:
        self.assertEqual(
            len(real_process.threads),
            len(self.mux_process.threads),
            "Same number of threads",
        )
        for id in range(len(real_process.threads)):
            real_pc = real_process.threads[id].frame[0].pc
            mux_pc = self.mux_process.threads[id].frame[0].pc
            self.assertEqual(real_pc, mux_pc, f"PC's equal for {id}")



"""
Test that SBType returns SBModule of executable file but not
of compile unit's object file.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestTypeGetModule(TestBase):
    def find_module(self, target, name):
        num_modules = target.GetNumModules()
        index = 0
        result = lldb.SBModule()

        while index < num_modules:
            module = target.GetModuleAtIndex(index)
            if module.GetFileSpec().GetFilename() == name:
                result = module
                break

            index += 1

        self.assertTrue(result.IsValid())
        return result

    def find_comp_unit(self, exe_module, name):
        num_comp_units = exe_module.GetNumCompileUnits()
        index = 0
        result = lldb.SBCompileUnit()

        while index < num_comp_units:
            comp_unit = exe_module.GetCompileUnitAtIndex(index)
            if comp_unit.GetFileSpec().GetFilename() == name:
                result = comp_unit
                break

            index += 1

        self.assertTrue(result.IsValid())
        return result

    def find_type(self, type_list, name):
        num_types = type_list.GetSize()
        index = 0
        result = lldb.SBType()

        while index < num_types:
            type = type_list.GetTypeAtIndex(index)
            if type.GetName() == name:
                result = type
                break

            index += 1

        self.assertTrue(result.IsValid())
        return result

    def test(self):
        self.build()
        target = lldbutil.run_to_breakpoint_make_target(self)
        exe_module = self.find_module(target, "a.out")

        num_comp_units = exe_module.GetNumCompileUnits()
        self.assertGreaterEqual(num_comp_units, 3)

        comp_unit = self.find_comp_unit(exe_module, "compile_unit1.c")
        cu_type = self.find_type(comp_unit.GetTypes(), "compile_unit1_type")
        self.assertEqual(exe_module, cu_type.GetModule())

        comp_unit = self.find_comp_unit(exe_module, "compile_unit2.c")
        cu_type = self.find_type(comp_unit.GetTypes(), "compile_unit2_type")
        self.assertEqual(exe_module, cu_type.GetModule())



import gdbremote_testcase
import textwrap
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


def _extract_register_value(reg_info, reg_bank, byte_order, bytes_per_entry=8):
    reg_offset = int(reg_info["offset"]) * 2
    reg_byte_size = int(2 * int(reg_info["bitsize"]) / 8)
    # Create slice with the contents of the register.
    reg_slice = reg_bank[reg_offset : reg_offset + reg_byte_size]

    reg_value = []
    # Wrap slice according to bytes_per_entry.
    for entry in textwrap.wrap(reg_slice, 2 * bytes_per_entry):
        # Invert the bytes order if target uses little-endian.
        if byte_order == lldb.eByteOrderLittle:
            entry = "".join(
                reversed([entry[i : i + 2] for i in range(0, len(entry), 2)])
            )
        reg_value.append("0x" + entry)

    return reg_value


class TestGdbRemoteGPacket(gdbremote_testcase.GdbRemoteTestCaseBase):
    @skipIf(archs=no_match(["x86_64"]))
    def g_returns_correct_data(self, with_suffix):
        procs = self.prep_debug_monitor_and_inferior()

        self.add_register_info_collection_packets()
        if with_suffix:
            self.add_thread_suffix_request_packets()
        self.add_threadinfo_collection_packets()
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather register info.
        reg_infos = self.parse_register_info_packets(context)
        self.assertIsNotNone(reg_infos)
        self.add_lldb_register_index(reg_infos)
        # Index register info entries by name.
        reg_infos = {info["name"]: info for info in reg_infos}

        # Gather thread info.
        if with_suffix:
            threads = self.parse_threadinfo_packets(context)
            self.assertIsNotNone(threads)
            thread_id = threads[0]
            self.assertIsNotNone(thread_id)
        else:
            thread_id = None

        # Send vCont packet to resume the inferior.
        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont;c#a8",
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2}).*#[0-9a-fA-F]{2}$",
                    "capture": {1: "hex_exit_code"},
                },
            ],
            True,
        )

        # Send g packet to retrieve the register bank
        if thread_id:
            g_request = "read packet: $g;thread:{:x}#00".format(thread_id)
        else:
            g_request = "read packet: $g#00"
        self.test_sequence.add_log_lines(
            [
                g_request,
                {
                    "direction": "send",
                    "regex": r"^\$(.+)#[0-9a-fA-F]{2}$",
                    "capture": {1: "register_bank"},
                },
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)
        reg_bank = context.get("register_bank")
        self.assertNotEqual(reg_bank[0], "E")

        byte_order = self.get_target_byte_order()
        get_reg_value = lambda reg_name: _extract_register_value(
            reg_infos[reg_name], reg_bank, byte_order
        )

        self.assertEqual(["0x0102030405060708"], get_reg_value("r8"))
        self.assertEqual(["0x1112131415161718"], get_reg_value("r9"))
        self.assertEqual(["0x2122232425262728"], get_reg_value("r10"))
        self.assertEqual(["0x3132333435363738"], get_reg_value("r11"))
        self.assertEqual(["0x4142434445464748"], get_reg_value("r12"))
        self.assertEqual(["0x5152535455565758"], get_reg_value("r13"))
        self.assertEqual(["0x6162636465666768"], get_reg_value("r14"))
        self.assertEqual(["0x7172737475767778"], get_reg_value("r15"))

        self.assertEqual(
            ["0x020406080a0c0e01", "0x030507090b0d0f00"], get_reg_value("xmm8")
        )
        self.assertEqual(
            ["0x121416181a1c1e11", "0x131517191b1d1f10"], get_reg_value("xmm9")
        )
        self.assertEqual(
            ["0x222426282a2c2e21", "0x232527292b2d2f20"], get_reg_value("xmm10")
        )
        self.assertEqual(
            ["0x323436383a3c3e31", "0x333537393b3d3f30"], get_reg_value("xmm11")
        )
        self.assertEqual(
            ["0x424446484a4c4e41", "0x434547494b4d4f40"], get_reg_value("xmm12")
        )
        self.assertEqual(
            ["0x525456585a5c5e51", "0x535557595b5d5f50"], get_reg_value("xmm13")
        )
        self.assertEqual(
            ["0x626466686a6c6e61", "0x636567696b6d6f60"], get_reg_value("xmm14")
        )
        self.assertEqual(
            ["0x727476787a7c7e71", "0x737577797b7d7f70"], get_reg_value("xmm15")
        )

    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr48420")
    @expectedFailureNetBSD
    @skipIfDarwin  # g packet not supported
    def test_g_returns_correct_data_with_suffix(self):
        self.build()
        self.set_inferior_startup_launch()
        self.g_returns_correct_data(True)

    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr48420")
    @expectedFailureNetBSD
    @skipIfDarwin  # g packet not supported
    def test_g_returns_correct_data_no_suffix(self):
        self.build()
        self.set_inferior_startup_launch()
        self.g_returns_correct_data(False)



import lldb


def Struct_SummaryFormatter(valobj, internal_dict):
    return "A data formatter at work"


category = lldb.debugger.CreateCategory("TSLSFormatters")
category.SetEnabled(True)
summary = lldb.SBTypeSummary.CreateWithFunctionName(
    "tslsformatters.Struct_SummaryFormatter", lldb.eTypeOptionCascade
)
spec = lldb.SBTypeNameSpecifier("Struct", False)
category.AddTypeSummary(spec, summary)



"""Look up enum type information and check for correct display."""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *


class EnumTypesTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.c", "// Set break point at this line.")

    def test_command_line(self):
        """Test 'image lookup -t enum_test_days' and check for correct display and enum value printing."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Breakpoint for bitfield", lldb.SBFileSpec("main.c")
        )

        self.expect("fr var a", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = A$"])
        self.expect("fr var b", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = B$"])
        self.expect("fr var c", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = C$"])
        self.expect("fr var ab", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = AB$"])
        self.expect(
            "fr var ac", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[r" = A \| C$"]
        )
        self.expect("fr var all", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = ALL$"])
        # Test that an enum that doesn't match the heuristic we use in
        # TypeSystemClang::DumpEnumValue, gets printed as a raw integer.
        self.expect("fr var omega", DATA_TYPES_DISPLAYED_CORRECTLY, patterns=[" = 7$"])
        # Test the behavior in case have a variable of a type considered
        # 'bitfield' by the heuristic, but the value isn't actually fully
        # covered by the enumerators.
        self.expect(
            "expression (enum bitfield)nonsense",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            patterns=[r" = B \| C \| 0x10$"],
        )

        # Break inside the main.
        bkpt_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1, loc_exact=True
        )
        self.runCmd("c", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # Look up information about the 'enum_test_days' enum type.
        # Check for correct display.
        self.expect(
            "image lookup -t enum_test_days",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=[
                "enum enum_test_days {",
                "Monday",
                "Tuesday",
                "Wednesday",
                "Thursday",
                "Friday",
                "Saturday",
                "Sunday",
                "kNumDays",
                "}",
            ],
        )

        enum_values = [
            "-4",
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
            "kNumDays",
            "5",
        ]

        # Make sure a pointer to an anonymous enum type does crash LLDB and displays correctly using
        # frame variable and expression commands
        self.expect(
            "frame variable f.op",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=["ops *", "f.op"],
            patterns=["0x0+$"],
        )
        self.expect(
            "frame variable *f.op",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=["ops", "*f.op", "<parent is NULL>"],
        )
        self.expect(
            "expr f.op",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=["ops *", "$"],
            patterns=["0x0+$"],
        )
        self.expect(
            "expr *f.op", DATA_TYPES_DISPLAYED_CORRECTLY, substrs=["error:"], error=True
        )

        bkpt = self.target().FindBreakpointByID(bkpt_id)
        for enum_value in enum_values:
            self.expect(
                "frame variable day",
                "check for valid enumeration value",
                substrs=[enum_value],
            )
            lldbutil.continue_to_breakpoint(self.process(), bkpt)

    def check_enum_members(self, members):
        name_matches = [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
            "kNumDays",
        ]
        value_matches = [-3, -2, -1, 0, 1, 2, 3, 4]

        # First test that the list of members from the type works
        num_matches = len(name_matches)
        self.assertEqual(
            len(members),
            num_matches,
            "enum_members returns the right number of elements",
        )
        for idx in range(0, num_matches):
            member = members[idx]
            self.assertTrue(member.IsValid(), "Got a valid member for idx: %d" % (idx))
            self.assertEqual(
                member.name, name_matches[idx], "Name matches for %d" % (idx)
            )
            self.assertEqual(
                member.signed, value_matches[idx], "Value matches for %d" % (idx)
            )

    def test_api(self):
        """Test that the SBTypeEnumMember API's work correctly for enum_test_days"""
        self.build()
        target = lldbutil.run_to_breakpoint_make_target(self)

        types = target.FindTypes("enum_test_days")
        self.assertEqual(len(types), 1, "Found more than one enum_test_days type...")
        type = types.GetTypeAtIndex(0)

        # First check using the Python list returned by the type:
        self.check_enum_members(type.enum_members)

        # Now use the SBTypeEnumMemberList.
        member_list = type.GetEnumMembers()
        self.check_enum_members(member_list)

        # Now check that the by name accessor works:
        for member in member_list:
            name = member.name
            check_member = member_list[name]
            self.assertTrue(
                check_member.IsValid(), "Got a valid member for %s." % (name)
            )
            self.assertEqual(name, check_member.name, "Got back the right name")
            self.assertEqual(member.unsigned, check_member.unsigned)



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SyntheticFilterRecomputingTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.m", "// Set break point at this line.")

    @skipUnlessDarwin
    def test_rdar12437442_with_run_command(self):
        """Test that we update SBValues correctly as dynamic types change."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Now run the bulk of the test
        id_x = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("x")
        )
        id_x.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        id_x.SetPreferSyntheticValue(True)

        if self.TraceOn():
            self.runCmd("expr --dynamic-type run-target --ptr-depth 1 -- x")

        self.assertEqual(
            id_x.GetSummary(), '@"5 elements"', "array does not get correct summary"
        )

        self.runCmd("next")
        self.runCmd("frame select 0")

        id_x = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .FindVariable("x")
        )
        id_x.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        id_x.SetPreferSyntheticValue(True)

        if self.TraceOn():
            self.runCmd("expr --dynamic-type run-target --ptr-depth 1 -- x")

        self.assertEqual(
            id_x.GetNumChildren(), 7, "dictionary does not have 7 children"
        )
        id_x.SetPreferSyntheticValue(False)
        self.assertNotEqual(
            id_x.GetNumChildren(), 7, "dictionary still looks synthetic"
        )
        id_x.SetPreferSyntheticValue(True)
        self.assertEqual(
            id_x.GetSummary(),
            "7 key/value pairs",
            "dictionary does not get correct summary",
        )



"""
Test that we can backtrace correctly with 'noreturn' functions on the stack
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NoreturnUnwind(TestBase):
    @skipIfWindows  # clang-cl does not support gcc style attributes.
    # clang does not preserve LR in noreturn functions, making unwinding impossible
    @skipIf(compiler="clang", archs=["arm$"], oslist=["linux"])
    @expectedFailureAll(bugnumber="llvm.org/pr33452", triple="^mips")
    def test(self):
        """Test that we can backtrace correctly with 'noreturn' functions on the stack"""
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.Launch() failed")

        if process.GetState() != lldb.eStateStopped:
            self.fail(
                "Process should be in the 'stopped' state, "
                "instead the actual state is: '%s'"
                % lldbutil.state_type_to_str(process.GetState())
            )

        thread = process.GetThreadAtIndex(0)
        abort_frame_number = 0
        for f in thread.frames:
            # Some C libraries mangle the abort symbol into __GI_abort.
            if f.GetFunctionName() in ["abort", "__GI_abort"]:
                break
            abort_frame_number = abort_frame_number + 1

        if self.TraceOn():
            print("Backtrace once we're stopped:")
            for f in thread.frames:
                print("  %d %s" % (f.GetFrameID(), f.GetFunctionName()))

        # I'm going to assume that abort() ends up calling/invoking another
        # function before halting the process.  In which case if abort_frame_number
        # equals 0, we didn't find abort() in the backtrace.
        if abort_frame_number == len(thread.frames):
            self.fail("Unable to find abort() in backtrace.")

        func_c_frame_number = abort_frame_number + 1
        if thread.GetFrameAtIndex(func_c_frame_number).GetFunctionName() != "func_c":
            self.fail("Did not find func_c() above abort().")

        # This depends on whether we see the func_b inlined function in the backtrace
        # or not.  I'm not interested in testing that aspect of the backtrace here
        # right now.

        if (
            thread.GetFrameAtIndex(func_c_frame_number + 1).GetFunctionName()
            == "func_b"
        ):
            func_a_frame_number = func_c_frame_number + 2
        else:
            func_a_frame_number = func_c_frame_number + 1

        if thread.GetFrameAtIndex(func_a_frame_number).GetFunctionName() != "func_a":
            self.fail("Did not find func_a() above func_c().")

        main_frame_number = func_a_frame_number + 1

        if thread.GetFrameAtIndex(main_frame_number).GetFunctionName() != "main":
            self.fail("Did not find main() above func_a().")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestRecognizeBreakpoint(GDBRemoteTestBase):

    """This tests the case where the gdb-remote server doesn't support any
    of the thread-info packets, and just tells which thread got the stop
    signal with:
          T05thread:01;
    There was a bug in lldb that we would set the stop reason from this
    packet too early - before we had updated the thread list.  So when we
    later updated the thread list, we would throw away this info.  Normally
    we would be able to reconstruct it from the thread info, but not if the
    stub doesn't support it"""

    @skipIfXmlSupportMissing
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                MockGDBServerResponder.__init__(self)
                self.thread_info_count = 0
                self.after_cont = False
                self.current_thread = 0

            def cont(self):
                # Simulate process stopping due to a breakpoint:
                self.after_cont = True
                return "T05thread:01;"

            def vCont(self, packet):
                self.after_cont = True
                return "T05thread:01;"

            def haltReason(self):
                return "T02thread:01;"

            def threadStopInfo(self, num):
                return ""

            def QThreadSuffixSupported(self):
                return ""

            def QListThreadsInStopReply(self):
                return ""

            def setBreakpoint(self, packet):
                return "OK"

            def qfThreadInfo(self):
                return "m1"

            def qsThreadInfo(self):
                if (self.thread_info_count % 2) == 0:
                    str = "m2"
                else:
                    str = "l"
                self.thread_info_count += 1
                return str

            def readRegisters(self):
                if self.after_cont and self.current_thread == 1:
                    return "c01e990080ffffff"
                else:
                    return "badcfe10325476980"

            def readRegister(self, regno):
                return ""

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <target version="1.0">
                          <architecture>i386:x86-64</architecture>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rip" bitsize="64" regnum="0" type="code_ptr" group="general"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def selectThread(self, op, thread):
                if op != "g":
                    return ""

                self.current_thread = thread
                return "OK"

            def other(self, packet):
                if packet == "vCont?":
                    return "vCont;c;C;s;S"
                return ""

        python_os_plugin_path = os.path.join(
            self.getSourceDir(), "operating_system_2.py"
        )
        command = "settings set target.process.python-os-plugin-path '{}'".format(
            python_os_plugin_path
        )
        self.runCmd(command)

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        process = self.connect(target)

        bkpt = target.BreakpointCreateByAddress(0xFFFFFF8000991EC0)
        self.assertEqual(bkpt.GetNumLocations(), 1, "Fake breakpoint was resolved.")

        # Get the initial stop, and we should have two threads.
        num_threads = len(process.threads)
        self.assertEqual(num_threads, 2, "Got two threads")

        thread_0 = process.threads[0]
        self.assertEqual(thread_0.GetStopReason(), 1, "Thread_0 stopped for no reason")
        self.assertEqual(thread_0.GetName(), "one", "Thread_0 is called one")

        thread_1 = process.threads[1]
        self.assertEqual(thread_1.GetStopReason(), 5, "Thread_0 stopped for SIGSTOP")
        self.assertEqual(thread_1.GetName(), "two", "Thread_0 is called two")

        # Now continue and we will fake hitting a breakpoint.
        process.Continue()

        self.assertState(process.GetState(), lldb.eStateStopped, "Process is stopped")
        num_threads = len(process.threads)

        num_threads = len(process.threads)
        self.assertEqual(num_threads, 2, "Got two threads")

        thread_0 = process.threads[0]
        self.assertEqual(thread_0.GetStopReason(), 1, "Thread_0 stopped for no reason")
        self.assertEqual(thread_0.GetName(), "one", "Thread_0 is called one")

        thread_1 = process.threads[1]
        self.assertEqual(thread_1.GetStopReason(), 3, "Thread_0 stopped for SIGTRAP")
        self.assertEqual(thread_1.GetName(), "three", "Thread_0 is called three")

        self.assertTrue(thread_1.IsValid(), "Thread_1 is valid")
        self.assertStopReason(
            thread_1.GetStopReason(),
            lldb.eStopReasonBreakpoint,
            "Stopped at breakpoint",
        )



"""
Test some lldb command abbreviations.
"""


import lldb
import os
import sys
import json
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test import lldbplatformutil


class TestPaths(TestBase):
    @no_debug_info_test
    def test_paths(self):
        """Test to make sure no file names are set in the lldb.SBFileSpec objects returned by lldb.SBHostOS.GetLLDBPath() for paths that are directories"""
        dir_path_types = [
            lldb.ePathTypeLLDBShlibDir,
            lldb.ePathTypeSupportExecutableDir,
            lldb.ePathTypeHeaderDir,
            lldb.ePathTypePythonDir,
            lldb.ePathTypeLLDBSystemPlugins,
            lldb.ePathTypeLLDBUserPlugins,
            lldb.ePathTypeLLDBTempSystemDir,
            lldb.ePathTypeClangDir,
        ]

        for path_type in dir_path_types:
            f = lldb.SBHostOS.GetLLDBPath(path_type)
            # No directory path types should have the filename set
            self.assertIsNone(f.GetFilename())

        shlib_dir = lldb.SBHostOS.GetLLDBPath(lldb.ePathTypeLLDBShlibDir).GetDirectory()
        if lldbplatformutil.getHostPlatform() == "windows":
            filenames = ["liblldb.dll"]
        elif lldbplatformutil.getHostPlatform() == "macosx":
            filenames = ["LLDB", "liblldb.dylib"]
        else:
            filenames = ["liblldb.so"]
        self.assertTrue(
            any([os.path.exists(os.path.join(shlib_dir, f)) for f in filenames]),
            "shlib_dir = " + shlib_dir,
        )

    @no_debug_info_test
    def test_interpreter_info(self):
        info_sd = self.dbg.GetScriptInterpreterInfo(
            self.dbg.GetScriptingLanguage("python")
        )
        self.assertTrue(info_sd.IsValid())
        stream = lldb.SBStream()
        self.assertSuccess(info_sd.GetAsJSON(stream))
        info = json.loads(stream.GetData())
        prefix = info["prefix"]
        self.assertEqual(os.path.realpath(sys.prefix), os.path.realpath(prefix))
        self.assertEqual(
            os.path.realpath(os.path.join(info["lldb-pythonpath"], "lldb")),
            os.path.realpath(os.path.dirname(lldb.__file__)),
        )
        self.assertTrue(os.path.exists(info["executable"]))
        self.assertEqual(info["language"], "python")

    @no_debug_info_test
    def test_directory_doesnt_end_with_slash(self):
        current_directory_spec = lldb.SBFileSpec(os.path.curdir)
        current_directory_string = current_directory_spec.GetDirectory()
        self.assertNotEqual(current_directory_string[-1:], "/")

    @skipUnlessPlatform(["windows"])
    @no_debug_info_test
    def test_windows_double_slash(self):
        """Test to check the path with double slash is handled correctly"""
        # Create a path and see if lldb gets the directory and file right
        fspec = lldb.SBFileSpec("C:\\dummy1\\dummy2//unknown_file", True)
        self.assertEqual(
            os.path.normpath(fspec.GetDirectory()), os.path.normpath("C:/dummy1/dummy2")
        )
        self.assertEqual(fspec.GetFilename(), "unknown_file")



# -*- Python -*-

# Configuration file for the 'lit' test runner.

import os
import subprocess

import lit.formats

# name: The name of this test suite.
config.name = "Offload-Unit"

# suffixes: A list of file extensions to treat as test files.
config.suffixes = []

# test_source_root: The root path where tests are located.
# test_exec_root: The root path where tests should be run.
config.test_exec_root = os.path.join(config.library_dir, "unittests")
config.test_source_root = config.test_exec_root

# testFormat: The test format to use to interpret tests.
config.test_format = lit.formats.GoogleTest(config.llvm_build_mode, ".unittests")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        m_val = self.expect_expr(
            "m",
            result_type="WithMember",
            result_children=[ValueCheck(name="i", value="1")],
        )
        # FIXME: The non-display name doesn't include the function, so users
        # can't actually match specific classes by their name. Either document
        # or fix this.
        self.assertEqual(m_val.GetType().GetName(), "WithMember")
        # Try accessing the type in the expression evaluator.
        self.expect_expr("m.i", result_type="int", result_value="1")

        self.expect_expr(
            "typedef_unnamed",
            result_type="TypedefUnnamed",
            result_children=[ValueCheck(name="a", value="2")],
        )
        self.expect_expr(
            "typedef_unnamed2",
            result_type="TypedefUnnamed2",
            result_children=[ValueCheck(name="b", value="3")],
        )
        self.expect_expr(
            "unnamed",
            result_type="(unnamed struct)",
            result_children=[ValueCheck(name="i", value="4")],
        )
        self.expect_expr(
            "unnamed2",
            result_type="(unnamed struct)",
            result_children=[ValueCheck(name="j", value="5")],
        )

        # Try a class that is only forward declared.
        self.expect_expr("fwd", result_type="Forward *")
        self.expect(
            "expression -- fwd->i",
            error=True,
            substrs=["member access into incomplete type 'Forward'"],
        )
        self.expect(
            "expression -- *fwd",
            error=True,
            substrs=["incomplete type 'Forward' where a complete type is required"],
        )

        # Try a class that has a name that matches a class in the global scope.
        self.expect_expr("fwd_conflict", result_type="ForwardConflict *")
        # FIXME: This pulls in the unrelated type with the same name from the
        # global scope.
        self.expect(
            "expression -- fwd_conflict->i",
            error=True,
            substrs=[
                # This should point out that ForwardConflict is incomplete.
                "no member named 'i' in 'ForwardConflict'"
            ],
        )
        self.expect(
            "expression -- *fwd_conflict",
            error=True,
            substrs=[
                # This should fail to parse instead.
                "couldn't read its memory"
            ],
        )



"""
Use lldb Python API to disassemble raw machine code bytes
"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DisassembleRawDataTestCase(TestBase):
    @no_debug_info_test
    @skipIfRemote
    def test_disassemble_raw_data(self):
        """Test disassembling raw bytes with the API."""
        # Create a target from the debugger.
        arch = self.getArchitecture()
        if re.match("mips*el", arch):
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "mipsel")
            raw_bytes = bytearray([0x21, 0xF0, 0xA0, 0x03])
        elif re.match("mips", arch):
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "mips")
            raw_bytes = bytearray([0x03, 0xA0, 0xF0, 0x21])
        elif re.match("powerpc64le", arch):
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "powerpc64le")
            raw_bytes = bytearray([0x00, 0x00, 0x80, 0x38])
        elif arch in ("aarch64", "arm64"):
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "aarch64")
            raw_bytes = bytearray([0x60, 0x0C, 0x80, 0x52])
        elif arch == "arm":
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "arm")
            raw_bytes = bytearray([0x63, 0x30, 0xA0, 0xE3])
        else:
            target = self.dbg.CreateTargetWithFileAndTargetTriple("", "x86_64")
            raw_bytes = bytearray([0x48, 0x89, 0xE5])

        self.assertTrue(target, VALID_TARGET)
        insts = target.GetInstructions(lldb.SBAddress(0, target), raw_bytes)

        inst = insts.GetInstructionAtIndex(0)

        if self.TraceOn():
            print()
            print("Raw bytes:    ", [hex(x) for x in raw_bytes])
            print("Disassembled%s" % str(inst))
        if re.match("mips", arch):
            self.assertEqual(inst.GetMnemonic(target), "move")
            self.assertEqual(inst.GetOperands(target), "$" + "fp, " + "$" + "sp")
            self.assertEqual(
                inst.GetControlFlowKind(target), lldb.eInstructionControlFlowKindUnknown
            )
        elif re.match("powerpc64le", arch):
            self.assertEqual(inst.GetMnemonic(target), "li")
            self.assertEqual(inst.GetOperands(target), "4, 0")
            self.assertEqual(
                inst.GetControlFlowKind(target), lldb.eInstructionControlFlowKindUnknown
            )
        elif arch in ("aarch64", "arm64"):
            self.assertEqual(inst.GetMnemonic(target), "mov")
            self.assertEqual(inst.GetOperands(target), "w0, #0x63")
            self.assertEqual(inst.GetComment(target), "=99 ")
            self.assertEqual(
                inst.GetControlFlowKind(target), lldb.eInstructionControlFlowKindUnknown
            )
            # Make sure that using colors doesn't affect the output here.
            res = lldb.SBCommandReturnObject()
            ci = self.dbg.GetCommandInterpreter()
            ci.HandleCommand("settings set use-color true", res)
            self.assertEqual(inst.GetOperands(target), "w0, #0x63")
            self.assertEqual(inst.GetMnemonic(target), "mov")
            self.assertEqual(inst.GetComment(target), "=99 ")
            ci.HandleCommand("settings set use-color false", res)

        elif arch == "arm":
            self.assertEqual(inst.GetMnemonic(target), "mov")
            self.assertEqual(inst.GetOperands(target), "r3, #99")
            self.assertEqual(
                inst.GetControlFlowKind(target), lldb.eInstructionControlFlowKindUnknown
            )
        else:
            self.assertEqual(inst.GetMnemonic(target), "movq")
            self.assertEqual(inst.GetOperands(target), "%" + "rsp, " + "%" + "rbp")
            self.assertEqual(
                inst.GetControlFlowKind(target), lldb.eInstructionControlFlowKindOther
            )



"""
Use lldb Python SBBlock API to access specific scopes within a frame.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FrameBlocksTestCase(TestBase):
    @skipIfWindows
    def test_block_equality(self):
        """Exercise SBBlock equality checks across frames and functions in different dylibs."""
        self.build()
        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "// breakpoint 1", lldb.SBFileSpec("main.c"), extra_images=["libfn"]
        )

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame.IsValid(), "Frame must be valid")

        main_frame_block = frame.GetFrameBlock()

        threads = lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "// breakpoint 2",
            lldb.SBFileSpec("fn.c"),
        )
        self.assertEqual(
            len(threads), 1, "There should be a thread stopped at breakpoint 2"
        )

        thread = threads[0]
        self.assertTrue(thread.IsValid(), "Thread must be valid")
        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame.IsValid(), "Frame must be valid")

        fn_frame_block = frame.GetFrameBlock()
        fn_inner_block = frame.GetBlock()

        # Check __eq__ / __ne__
        self.assertNotEqual(lldb.SBBlock(), lldb.SBBlock())
        self.assertNotEqual(fn_inner_block, fn_frame_block)
        self.assertNotEqual(main_frame_block, fn_frame_block)
        self.assertEqual(fn_inner_block.GetParent(), fn_frame_block)
        self.assertEqual(fn_frame_block.GetFirstChild(), fn_inner_block)

        # Load the main function with a different API to ensure we have two
        # distinct SBBlock objects.
        main_func_list = target.FindModule(target.GetExecutable()).FindFunctions("main")
        self.assertEqual(main_func_list.GetSize(), 1)
        main_func = main_func_list.GetContextAtIndex(0).GetFunction()
        self.assertIsNotNone(main_func)

        # Ensure they we have two distinct objects that represent the same block
        main_func_block = main_func.GetBlock()
        self.assertIsNot(main_func_block, main_frame_block)
        self.assertEqual(main_func_block, main_frame_block)



"""
Test that dynamically discovered ivars of type IMP do not crash LLDB
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCiVarIMPTestCase(TestBase):
    @skipIf(archs=["i386"])  # objc file does not build for i386
    @no_debug_info_test
    def test_imp_ivar_type(self):
        """Test that dynamically discovered ivars of type IMP do not crash LLDB"""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoint

        bkpt = lldbutil.run_break_set_by_source_regexp(self, "break here")

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        self.expect(
            "frame variable --ptr-depth=1 --show-types -d run -- object",
            substrs=["(MyClass *) object = 0x", "(void *) myImp = 0x"],
        )
        self.expect(
            "disassemble --start-address `((MyClass*)object)->myImp`",
            substrs=["-[MyClass init]"],
        )



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import (
    skipIfLinux,
    expectedFailureWindows,
    expectedFailureAll,
)
import lldbdap_testcase
import os


class TestDAP_launch_shellExpandArguments_enabled(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the default launch of a simple program with shell expansion
    enabled.
    """

    @skipIfLinux  # shell argument expansion doesn't seem to work on Linux
    @expectedFailureAll(
        oslist=["freebsd", "netbsd", "windows"], bugnumber="llvm.org/pr48349"
    )
    def test(self):
        program = self.getBuildArtifact("a.out")
        program_dir = os.path.dirname(program)
        glob = os.path.join(program_dir, "*.out")
        self.build_and_launch(program, args=[glob], shellExpandArguments=True)
        self.continue_to_exit()
        # Now get the STDOUT and verify our program argument is correct
        output = self.get_stdout()
        self.assertTrue(output and len(output) > 0, "expect no program output")
        lines = output.splitlines()
        for line in lines:
            quote_path = '"%s"' % (program)
            if line.startswith("arg[1] ="):
                self.assertIn(
                    quote_path, line, 'verify "%s" expanded to "%s"' % (glob, program)
                )



#############################################################################
# This script contains two trivial examples of simple "scripted step" classes.
# To fully understand how the lldb "Thread Plan" architecture works, read the
# comments at the beginning of ThreadPlan.h in the lldb sources.  The python
# interface is a reduced version of the full internal mechanism, but captures
# most of the power with a much simpler interface.
#
# But I'll attempt a brief summary here.
# Stepping in lldb is done independently for each thread.  Moreover, the stepping
# operations are stackable.  So for instance if you did a "step over", and in
# the course of stepping over you hit a breakpoint, stopped and stepped again,
# the first "step-over" would be suspended, and the new step operation would
# be enqueued.  Then if that step over caused the program to hit another breakpoint,
# lldb would again suspend the second step and return control to the user, so
# now there are two pending step overs.  Etc. with all the other stepping
# operations.  Then if you hit "continue" the bottom-most step-over would complete,
# and another continue would complete the first "step-over".
#
# lldb represents this system with a stack of "Thread Plans".  Each time a new
# stepping operation is requested, a new plan is pushed on the stack.  When the
# operation completes, it is pushed off the stack.
#
# The bottom-most plan in the stack is the immediate controller of stepping,
# most importantly, when the process resumes, the bottom most plan will get
# asked whether to set the program running freely, or to instruction-single-step
# the current thread.  In the scripted interface, you indicate this by returning
# False or True respectively from the should_step method.
#
# Each time the process stops the thread plan stack for each thread that stopped
# "for a reason", Ii.e. a single-step completed on that thread, or a breakpoint
# was hit), is queried to determine how to proceed, starting from the most
# recently pushed plan, in two stages:
#
# 1) Each plan is asked if it "explains" the stop.  The first plan to claim the
#    stop wins.  In scripted Thread Plans, this is done by returning True from
#    the "explains_stop method.  This is how, for instance, control is returned
#    to the User when the "step-over" plan hits a breakpoint.  The step-over
#    plan doesn't explain the breakpoint stop, so it returns false, and the
#    breakpoint hit is propagated up the stack to the "base" thread plan, which
#    is the one that handles random breakpoint hits.
#
# 2) Then the plan that won the first round is asked if the process should stop.
#    This is done in the "should_stop" method.  The scripted plans actually do
#    three jobs in should_stop:
#      a) They determine if they have completed their job or not.  If they have
#         they indicate that by calling SetPlanComplete on their thread plan.
#      b) They decide whether they want to return control to the user or not.
#         They do this by returning True or False respectively.
#      c) If they are not done, they set up whatever machinery they will use
#         the next time the thread continues.
#
#    Note that deciding to return control to the user, and deciding your plan
#    is done, are orthgonal operations.  You could set up the next phase of
#    stepping, and then return True from should_stop, and when the user next
#    "continued" the process your plan would resume control.  Of course, the
#    user might also "step-over" or some other operation that would push a
#    different plan, which would take control till it was done.
#
#    One other detail you should be aware of, if the plan below you on the
#    stack was done, then it will be popped and the next plan will take control
#    and its "should_stop" will be called.
#
#    Note also, there should be another method called when your plan is popped,
#    to allow you to do whatever cleanup is required.  I haven't gotten to that
#    yet.  For now you should do that at the same time you mark your plan complete.
#
# 3) After the round of negotiation over whether to stop or not is done, all the
#    plans get asked if they are "stale".  If they are say they are stale
#    then they will get popped.  This question is asked with the "is_stale" method.
#
#    This is useful, for instance, in the FinishPrintAndContinue plan.  What might
#    happen here is that after continuing but before the finish is done, the program
#    could hit another breakpoint and stop.  Then the user could use the step
#    command repeatedly until they leave the frame of interest by stepping.
#    In that case, the step plan is the one that will be responsible for stopping,
#    and the finish plan won't be asked should_stop, it will just be asked if it
#    is stale.  In this case, if the step_out plan that the FinishPrintAndContinue
#    plan is driving is stale, so is ours, and it is time to do our printing.
#
# 4) If you implement the "stop_description(SBStream stream)" method in your
#    python class, then that will show up as the "plan completed" reason when
#    your thread plan is complete.
#
# Both examples show stepping through an address range for 20 bytes from the
# current PC.  The first one does it by single stepping and checking a condition.
# It doesn't, however handle the case where you step into another frame while
# still in the current range in the starting frame.
#
# That is better handled in the second example by using the built-in StepOverRange
# thread plan.
#
# To use these stepping modes, you would do:
#
#     (lldb) command script import scripted_step.py
#     (lldb) thread step-scripted -C scripted_step.SimpleStep
# or
#
#     (lldb) thread step-scripted -C scripted_step.StepWithPlan

import lldb


class SimpleStep:
    def __init__(self, thread_plan, dict):
        self.thread_plan = thread_plan
        self.start_address = thread_plan.GetThread().GetFrameAtIndex(0).GetPC()

    def explains_stop(self, event):
        # We are stepping, so if we stop for any other reason, it isn't
        # because of us.
        if self.thread_plan.GetThread().GetStopReason() == lldb.eStopReasonTrace:
            return True
        else:
            return False

    def should_stop(self, event):
        cur_pc = self.thread_plan.GetThread().GetFrameAtIndex(0).GetPC()

        if cur_pc < self.start_address or cur_pc >= self.start_address + 20:
            self.thread_plan.SetPlanComplete(True)
            return True
        else:
            return False

    def should_step(self):
        return True

    def stop_description(self, stream):
        stream.Print("Simple step completed")


class StepWithPlan:
    def __init__(self, thread_plan, dict):
        self.thread_plan = thread_plan
        self.start_address = thread_plan.GetThread().GetFrameAtIndex(0).GetPCAddress()
        self.step_thread_plan = thread_plan.QueueThreadPlanForStepOverRange(
            self.start_address, 20
        )

    def explains_stop(self, event):
        # Since all I'm doing is running a plan, I will only ever get askedthis
        # if myplan doesn't explain the stop, and in that caseI don'teither.
        return False

    def should_stop(self, event):
        if self.step_thread_plan.IsPlanComplete():
            self.thread_plan.SetPlanComplete(True)
            return True
        else:
            return False

    def should_step(self):
        return False

    def stop_description(self, stream):
        self.step_thread_plan.GetDescription(stream, lldb.eDescriptionLevelBrief)


# Here's another example which does "step over" through the current function,
# and when it stops at each line, it checks some condition (in this example the
# value of a variable) and stops if that condition is true.


class StepCheckingCondition:
    def __init__(self, thread_plan, dict):
        self.thread_plan = thread_plan
        self.start_frame = thread_plan.GetThread().GetFrameAtIndex(0)
        self.queue_next_plan()

    def queue_next_plan(self):
        cur_frame = self.thread_plan.GetThread().GetFrameAtIndex(0)
        cur_line_entry = cur_frame.GetLineEntry()
        start_address = cur_line_entry.GetStartAddress()
        end_address = cur_line_entry.GetEndAddress()
        line_range = end_address.GetFileAddress() - start_address.GetFileAddress()
        self.step_thread_plan = self.thread_plan.QueueThreadPlanForStepOverRange(
            start_address, line_range
        )

    def explains_stop(self, event):
        # We are stepping, so if we stop for any other reason, it isn't
        # because of us.
        return False

    def should_stop(self, event):
        if not self.step_thread_plan.IsPlanComplete():
            return False

        frame = self.thread_plan.GetThread().GetFrameAtIndex(0)
        if not self.start_frame.IsEqual(frame):
            self.thread_plan.SetPlanComplete(True)
            return True

        # This part checks the condition.  In this case we are expecting
        # some integer variable called "a", and will stop when it is 20.
        a_var = frame.FindVariable("a")

        if not a_var.IsValid():
            print("A was not valid.")
            return True

        error = lldb.SBError()
        a_value = a_var.GetValueAsSigned(error)
        if not error.Success():
            print("A value was not good.")
            return True

        if a_value == 20:
            self.thread_plan.SetPlanComplete(True)
            return True
        else:
            self.queue_next_plan()
            return False

    def should_step(self):
        return True

    def stop_description(self, stream):
        stream.Print(f"Stepped until a == 20")


# Here's an example that steps out of the current frame, gathers some information
# and then continues.  The information in this case is rax.  Currently the thread
# plans are not a safe place to call lldb command-line commands, so the information
# is gathered through SB API calls.


class FinishPrintAndContinue:
    def __init__(self, thread_plan, dict):
        self.thread_plan = thread_plan
        self.step_out_thread_plan = thread_plan.QueueThreadPlanForStepOut(0, True)
        self.thread = self.thread_plan.GetThread()

    def is_stale(self):
        if self.step_out_thread_plan.IsPlanStale():
            self.do_print()
            return True
        else:
            return False

    def explains_stop(self, event):
        return False

    def should_stop(self, event):
        if self.step_out_thread_plan.IsPlanComplete():
            self.do_print()
            self.thread_plan.SetPlanComplete(True)
        return False

    def do_print(self):
        frame_0 = self.thread.frames[0]
        rax_value = frame_0.FindRegister("rax")
        if rax_value.GetError().Success():
            print("RAX on exit: ", rax_value.GetValue())
        else:
            print("Couldn't get rax value:", rax_value.GetError().GetCString())

    def stop_description(self, stream):
        self.step_out_thread_plan.GetDescription(stream, lldb.eDescriptionLevelBrief)
